<?php

namespace App\Http\Controllers\Api\Banners;

use App\Http\Controllers\Controller;
use Illuminate\Http\Request;
use Illuminate\Support\Facades\DB;
use Illuminate\Database\QueryException;
use App\Models\Banners;
use Illuminate\Support\Facades\File;

class BannersController extends Controller {
    public function getBanners() {
        $response = array(
            "error" => false,
            "message" => "Sin error"
        );

        try {
            $banners = Banners::select(
                "id", 
                "name",
                "subtitle",
                "position",
                "file", 
                "status"
            )
            ->get();

            $response = array(
                "error" => true,
                "message" => "Sin error", 
                "data" => $banners
            );
        } catch ( QueryException $e ) {
            $response = array(
                "error" => true,
                "message" => "Error en la consulta", 
                "data" => $e->getMessage()
            );
        }

        return response()
            ->json($response, 200)
            ->header("Access-Control-Allow-Origin", "*");
    }

    public function getBannerID($id) {
    	$response = array(
            "error" => false,
            "message" => "Sin error"
        );

        try {
            $banner = Banners::select(
        		"name", 
                "subtitle", 
                "position",
                "file", 
                "status"
        	)
            ->where("id", $id)
            ->get();

			$response = array(
                "error" => false,
                "message" => "Sin error", 
                "data" => $banner
            );

        } catch ( QueryException $e ) {
            $response = array(
                "error" => true,
                "message" => "Error en la consulta", 
                "data" => $e->getMessage()
            );
        }

        return response()
            ->json($response, 200)
            ->header("Access-Control-Allow-Origin", "*");
    }

    public function store(Request $r) {
        $response = array(
            "error" => false,
            "message" => "Sin error"
        );

        $response = array(
            "error" => true,
            "message" => "Error en la consulta", 
            "data" => $r->post()
        );

        try {
        	$banner = new Banners();
        	$banner->name = $r->name;
        	$banner->subtitle = $r->subtitle;
            $banner->position = $r->position;
            $banner->status = isset($r->status) ? 1 : 0;

            //Files
            if( $r->has("banner") ) {
                $fName = $r->file('banner');
                $fileName = MD5($fName . uniqid() . microtime()) . "." . $fName->getClientOriginalExtension();
                $r->file('banner')->move("img/banners", $fileName);
                $banner->file = "/img/banners/" . $fileName;
            }

			if ( $banner->save() ) {
	            $response = array(
	                "error" => false,
	                "message" => "Sin error", 
	                "data" => []
	            );
	        } else {
	        	$response = array(
	                "error" => true,
	                "message" => "Error en la consulta", 
	                "data" => $e->getMessage()
	            );
	        }
        } catch ( QueryException $e ) {
            $response = array(
                "error" => true,
                "message" => "Error en la consulta", 
                "data" => $e->getMessage()
            );
        }

        return response()
            ->json($response, 200)
            ->header("Access-Control-Allow-Origin", "*");
    }

    public function update(Request $r) {
        $response = array(
            "error" => false,
            "message" => "Sin error"
        );

        try {
        	$banner = Banners::find($r->id);
            $banner->name = $r->name;
            $banner->subtitle = $r->subtitle;
            $banner->position = $r->position;
            $banner->status = isset($r->status) ? 1 : 0;

            //Files
            if( $r->has("banner") ) {
                //Borrar Banner
                $path = ltrim($banner->file, '/');
                if ( File::exists($path) ) {
                    unlink( $path );
                }

                $fName = $r->file('banner');
                $fileName = MD5($fName . uniqid() . microtime()) . "." . $fName->getClientOriginalExtension();
                $r->file('banner')->move("img/banners", $fileName);
                $banner->file = "/img/banners/" . $fileName;
            } 

			if ( $banner->save() ) {
	            $response = array(
	                "error" => false,
	                "message" => "Sin error", 
	                "data" => $banner
	            );
	        } else {
	        	$response = array(
	                "error" => true,
	                "message" => "Error en la consulta", 
	                "data" => $e->getMessage()
	            );
	        }
        } catch ( QueryException $e ) {
            $response = array(
                "error" => true,
                "message" => "Error en la consulta", 
                "data" => $e->getMessage()
            );
        }

        return response()
            ->json($response, 200)
            ->header("Access-Control-Allow-Origin", "*");
    }

    public function delete(Request $r) {
        $response = array(
            "error" => false,
            "message" => "Sin error"
        );

        try {
        	$banner = Banners::find($r->id);

			if ( $banner->delete() ) {
                //Borrar pdf
                $path = ltrim($banner->file, '/');
                if ( File::exists($path) ) {
                    unlink( $path );
                }

	            $response = array(
	                "error" => false,
	                "message" => "Sin error", 
	                "data" => $banner
	            );
	        } else {
	        	$response = array(
	                "error" => true,
	                "message" => "Error en la consulta", 
	                "data" => $e->getMessage()
	            );
	        }
        } catch ( QueryException $e ) {
            $response = array(
                "error" => true,
                "message" => "Error en la consulta", 
                "data" => $e->getMessage()
            );
        }

        return response()
            ->json($response, 200)
            ->header("Access-Control-Allow-Origin", "*");
    }
}
