<?php

namespace App\Http\Controllers\Api\Downloads;

use App\Http\Controllers\Controller;
use Illuminate\Http\Request;
use Illuminate\Support\Facades\DB;
use Illuminate\Database\QueryException;
use App\Models\Downloads;
use Illuminate\Support\Facades\File;

class DownloadsController extends Controller {
    public function getDownloads() {
        $response = array(
            "error" => false,
            "message" => "Sin error"
        );

        try {
            $downloads = Downloads::select(
                "id", 
                "name",
                "desc", 
                "status"
            )
            ->get();

            $response = array(
                "error" => true,
                "message" => "Sin error", 
                "data" => $downloads
            );
        } catch ( QueryException $e ) {
            $response = array(
                "error" => true,
                "message" => "Error en la consulta", 
                "data" => $e->getMessage()
            );
        }

        return response()
            ->json($response, 200)
            ->header("Access-Control-Allow-Origin", "*");
    }

    public function getDownloadID($id) {
    	$response = array(
            "error" => false,
            "message" => "Sin error"
        );

        try {
            $download = Downloads::select(
        		"name", 
                "desc", 
                "file", 
                "image", 
                "status"
        	)
            ->where("id", $id)
            ->get();

			$response = array(
                "error" => false,
                "message" => "Sin error", 
                "data" => $download
            );

        } catch ( QueryException $e ) {
            $response = array(
                "error" => true,
                "message" => "Error en la consulta", 
                "data" => $e->getMessage()
            );
        }

        return response()
            ->json($response, 200)
            ->header("Access-Control-Allow-Origin", "*");
    }

    public function store(Request $r) {
        $response = array(
            "error" => false,
            "message" => "Sin error"
        );

        $response = array(
            "error" => true,
            "message" => "Error en la consulta", 
            "data" => $r->post()
        );

        try {
        	$download = new Downloads();
        	$download->name = $r->name;
        	$download->desc = $r->desc;
            $download->status = isset($r->status) ? 1 : 0;

            //Files
            if( $r->has("pdf") ) {
                $fName = $r->file('pdf');
                $fileName = MD5($fName . uniqid() . microtime()) . "." . $fName->getClientOriginalExtension();
                $r->file('pdf')->move("pdf/downloads", $fileName);
                $download->file = "/pdf/downloads/" . $fileName;
            }

            //Image
            if( $r->has("image") ) {
                $fName = $r->file('image');
                $fileName = MD5($fName . uniqid() . microtime()) . "." . $fName->getClientOriginalExtension();
                $r->file('image')->move("image/downloads", $fileName);
                $download->image = "/image/downloads/" . $fileName;
            }

			if ( $download->save() ) {
	            $response = array(
	                "error" => false,
	                "message" => "Sin error", 
	                "data" => []
	            );
	        } else {
	        	$response = array(
	                "error" => true,
	                "message" => "Error en la consulta", 
	                "data" => $e->getMessage()
	            );
	        }
        } catch ( QueryException $e ) {
            $response = array(
                "error" => true,
                "message" => "Error en la consulta", 
                "data" => $e->getMessage()
            );
        }

        return response()
            ->json($response, 200)
            ->header("Access-Control-Allow-Origin", "*");
    }

    public function update(Request $r) {
        $response = array(
            "error" => false,
            "message" => "Sin error"
        );

        try {
        	$download = Downloads::find($r->id);
            $download->name = $r->name;
            $download->desc = $r->desc;
            $download->status = isset($r->status) ? 1 : 0;

            //Files
            if( $r->has("pdf") ) {
                //Borrar pdf
                $path = ltrim($download->file, '/');
                if ( File::exists($path) ) {
                    unlink( $path );
                }

                $fName = $r->file('pdf');
                $fileName = MD5($fName . uniqid() . microtime()) . "." . $fName->getClientOriginalExtension();
                $r->file('pdf')->move("pdf/downloads", $fileName);
                $download->file = "/pdf/downloads/" . $fileName;
            } 

            //Images
            if( $r->has("image") ) {
                //Borrar image
                $path = ltrim($download->image, '/');
                if ( File::exists($path) ) {
                    unlink( $path );
                }

                $fName = $r->file('image');
                $fileName = MD5($fName . uniqid() . microtime()) . "." . $fName->getClientOriginalExtension();
                $r->file('image')->move("image/downloads", $fileName);
                $download->image = "/image/downloads/" . $fileName;
            } 

			if ( $download->save() ) {
	            $response = array(
	                "error" => false,
	                "message" => "Sin error", 
	                "data" => $download
	            );
	        } else {
	        	$response = array(
	                "error" => true,
	                "message" => "Error en la consulta", 
	                "data" => $e->getMessage()
	            );
	        }
        } catch ( QueryException $e ) {
            $response = array(
                "error" => true,
                "message" => "Error en la consulta", 
                "data" => $e->getMessage()
            );
        }

        return response()
            ->json($response, 200)
            ->header("Access-Control-Allow-Origin", "*");
    }

    public function delete(Request $r) {
        $response = array(
            "error" => false,
            "message" => "Sin error"
        );

        try {
        	$download = Downloads::find($r->id);

			if ( $download->delete() ) {
                //Borrar pdf
                $path = ltrim($download->file, '/');
                if ( File::exists($path) ) {
                    unlink( $path );
                }

                //Borrar image
                $path = ltrim($download->image, '/');
                if ( File::exists($path) ) {
                    unlink( $path );
                }

	            $response = array(
	                "error" => false,
	                "message" => "Sin error", 
	                "data" => $download
	            );
	        } else {
	        	$response = array(
	                "error" => true,
	                "message" => "Error en la consulta", 
	                "data" => $e->getMessage()
	            );
	        }
        } catch ( QueryException $e ) {
            $response = array(
                "error" => true,
                "message" => "Error en la consulta", 
                "data" => $e->getMessage()
            );
        }

        return response()
            ->json($response, 200)
            ->header("Access-Control-Allow-Origin", "*");
    }
}
