<?php

namespace App\Http\Controllers\Api\Ingredients;

use App\Http\Controllers\Controller;
use Illuminate\Http\Request;
use Illuminate\Support\Facades\DB;
use Illuminate\Database\QueryException;
use App\Models\CatUnits;
use App\Models\CatIngredients;
use App\Models\Ingredients;

class IngredientsController extends Controller {
    public function getIngredients() {
        $response = array(
            "error" => false,
            "message" => "Sin error"
        );

        try {
            $ingredients = DB::table("ingredientes")
            ->select(
                "ingredientes.id", 
                "ingredientes.ingredient", 
                DB::Raw("cat_ingredientes.name as ingredientName"),
                "ingredientes.name", 
                "ingredientes.portion", 
                "ingredientes.unit", 
                DB::Raw("cat_unidad.name as unitName"),
                "ingredientes.grams", 
                "ingredientes.calories", 
                "ingredientes.protein", 
                "ingredientes.fat", 
                "ingredientes.carbs", 
                "ingredientes.allergens", 
                "ingredientes.tip", 
                "ingredientes.status"
            )
            ->join("cat_ingredientes", "ingredientes.ingredient", "=", "cat_ingredientes.id")
            ->join("cat_unidad", "ingredientes.unit", "=", "cat_unidad.id")
            ->whereNull("ingredientes.deleted_at")
            ->whereNull("cat_ingredientes.deleted_at")
            ->whereNull("cat_unidad.deleted_at")
            ->orderBy("ingredientes.name")
            ->get();

            /*$ingredients = DB::table("ingredientes")
            ->select(
                "ingredientes.id", 
                "ingredientes.name", 
                "ingredientes.portion", 
                DB::Raw("cat_unidad.name as unitName"),
            )
            ->join("cat_ingredientes", "ingredientes.ingredient", "=", "cat_ingredientes.id")
            ->join("cat_unidad", "ingredientes.unit", "=", "cat_unidad.id")
            ->whereNull("ingredientes.deleted_at")
            ->whereNull("cat_ingredientes.deleted_at")
            ->whereNull("cat_unidad.deleted_at")
            ->get();*/

            $response = array(
                "error" => true,
                "message" => "Sin error", 
                "data" => $ingredients
            );
        } catch ( QueryException $e ) {
            $response = array(
                "error" => true,
                "message" => "Error en la consulta", 
                "data" => $e->getMessage()
            );
        }

        return response()
            ->json($response, 200)
            ->header("Access-Control-Allow-Origin", "*");
    }

    public function getCatIngredients() {
        $response = array(
            "error" => false,
            "message" => "Sin error"
        );

        try {
            $catIngredients = CatIngredients::select("id", "name")->orderBy("name")->get();

            $response = array(
                "error" => true,
                "message" => "Sin error", 
                "data" => $catIngredients
            );
        } catch ( QueryException $e ) {
            $response = array(
                "error" => true,
                "message" => "Error en la consulta", 
                "data" => $e->getMessage()
            );
        }

        return response()
            ->json($response, 200)
            ->header("Access-Control-Allow-Origin", "*");
    }

    public function getCatUnits() {
        $response = array(
            "error" => false,
            "message" => "Sin error"
        );

        try {
            $catUnits = CatUnits::select("id", "name")->orderBy("name")->get();

            $response = array(
                "error" => true,
                "message" => "Sin error", 
                "data" => $catUnits
            );
        } catch ( QueryException $e ) {
            $response = array(
                "error" => true,
                "message" => "Error en la consulta", 
                "data" => $e->getMessage()
            );
        }

        return response()
            ->json($response, 200)
            ->header("Access-Control-Allow-Origin", "*");
    }

    public function getIngredientID($id) {
        $response = array(
            "error" => false,
            "message" => "Sin error"
        );

        try {
            $ingredient = Ingredients::select("ingredient", "name", "portion", "unit", "grams", "calories", "protein", "fat", "carbs", "allergens", "tip", "status")->where("id", $id)->get();

            $response = array(
                "error" => false,
                "message" => "Sin error", 
                "data" => $ingredient
            );

        } catch ( QueryException $e ) {
            $response = array(
                "error" => true,
                "message" => "Error en la consulta", 
                "data" => $e->getMessage()
            );
        }

        return response()
            ->json($response, 200)
            ->header("Access-Control-Allow-Origin", "*");
    }

    public function store(Request $r) {
        $response = array(
            "error" => false,
            "message" => "Sin error"
        );

        try {
            $ingredient = new Ingredients();
            $ingredient->ingredient = $r->categoria;
            $ingredient->name = $r->alimento;
            $ingredient->portion = $r->porcion;
            $ingredient->unit = $r->unidad;
            $ingredient->grams = $r->gramos;
            $ingredient->calories = $r->calorias;
            $ingredient->protein = $r->proteina;
            $ingredient->fat = $r->grasa;
            $ingredient->carbs = $r->carbohidratos;
            $ingredient->allergens = $r->alergenos == "" ? " " : $r->alergenos;
            $ingredient->tip = $r->tips == "" ? " " : $r->tips;
            $ingredient->status = isset($r->status) ? 1 : 0;

            if ( $ingredient->save() ) {
                $response = array(
                    "error" => false,
                    "message" => "Sin error", 
                    "data" => $ingredient
                );
            } else {
                $response = array(
                    "error" => true,
                    "message" => "Error en la consulta", 
                    "data" => $e->getMessage()
                );
            }
        } catch ( QueryException $e ) {
            $response = array(
                "error" => true,
                "message" => "Error en la consulta", 
                "data" => $e->getMessage()
            );
        }

        return response()
            ->json($response, 200)
            ->header("Access-Control-Allow-Origin", "*");
    }

    public function update(Request $r) {
        $response = array(
            "error" => false,
            "message" => "Sin error"
        );

        try {
            $ingredient = Ingredients::find($r->id);
            $ingredient->ingredient = $r->categoria;
            $ingredient->name = $r->alimento;
            $ingredient->portion = $r->porcion;
            $ingredient->unit = $r->unidad;
            $ingredient->grams = $r->gramos;
            $ingredient->calories = $r->calorias;
            $ingredient->protein = $r->proteina;
            $ingredient->fat = $r->grasa;
            $ingredient->carbs = $r->carbohidratos;
            $ingredient->allergens = $r->alergenos == "" ? " " : $r->alergenos;
            $ingredient->tip = $r->tips == "" ? " " : $r->tips;
            $ingredient->status = isset($r->status) ? 1 : 0;
            
            if ( $ingredient->save() ) {
                $response = array(
                    "error" => false,
                    "message" => "Sin error", 
                    "data" => $ingredient
                );
            } else {
                $response = array(
                    "error" => true,
                    "message" => "Error en la consulta", 
                    "data" => $e->getMessage()
                );
            }
        } catch ( QueryException $e ) {
            $response = array(
                "error" => true,
                "message" => "Error en la consulta", 
                "data" => $e->getMessage()
            );
        }

        return response()
            ->json($response, 200)
            ->header("Access-Control-Allow-Origin", "*");
    }

    public function delete(Request $r) {
        $response = array(
            "error" => false,
            "message" => "Sin error"
        );

        try {
            $ingredient = Ingredients::find($r->id);

            if ( $ingredient->delete() ) {
                $response = array(
                    "error" => false,
                    "message" => "Sin error", 
                    "data" => $ingredient
                );
            } else {
                $response = array(
                    "error" => true,
                    "message" => "Error en la consulta", 
                    "data" => $e->getMessage()
                );
            }
        } catch ( QueryException $e ) {
            $response = array(
                "error" => true,
                "message" => "Error en la consulta", 
                "data" => $e->getMessage()
            );
        }

        return response()
            ->json($response, 200)
            ->header("Access-Control-Allow-Origin", "*");
    }
}
