<?php
namespace App\Http\Controllers\Api\Integraciones\Login;

use App\Http\Controllers\Controller;
use Illuminate\Http\Request;
use Tymon\JWTAuth\Facades\JWTAuth;
use Tymon\JWTAuth\Exceptions\JWTException;
use Illuminate\Support\Facades\DB;
use Illuminate\Database\QueryException;
use App\Models\Clients;
use App\Models\Programs;
use App\Models\Workouts;
use Illuminate\Support\Facades\Hash;

use PHPMailer\PHPMailer\PHPMailer;
use PHPMailer\PHPMailer\SMTP;
use PHPMailer\PHPMailer\Exception;

use \Firebase\JWT\JWT;
use DateTime;


class AuthController extends Controller {
    public function login( Request $r ) {
        if ( Request()->header('x-glowme-key') != $_ENV['X_GLOWME_API'] ) { 
            return response()
            ->json([ 
                'error' => true, 
                'message' => "Api key error", 
                'data' => "Acceso no permitido" 
            ], 403)
            ->header("Access-Control-Allow-Origin", "*");
        } else {
            try {
                $cliente = Clients::where('email', $r->email)->first();

                if ($cliente && Hash::check($r->password, $cliente->password)) {
                    $token = JWTAuth::fromUser($cliente);

                    $Programs = Programs::select('name')->where("id", $cliente->nutrition_plan)->first();
                    $Workouts = Workouts::select('program')->where("id", $cliente->workout_plan)->first();

                    if ( $Programs === null ) {
                        $Programs['name'] = "";
                    }

                    if ( $Workouts === null ) {
                        $Workouts['program'] = "";
                    }

                    if ( $cliente['status'] == 1 ) {
                        if ( $cliente['last_charge'] == null ) {
                            $fecha1 = new DateTime(date("Y-m-d", strtotime($cliente['created_at'])));
                            $fecha2 = new DateTime(date("Y-m-d"));
                            $diff = $fecha1->diff($fecha2);
                            switch ( $cliente['suscription_plan'] ) {
                                case 1: 
                                    $dias = 29;
                                    break;
                                case 2:
                                    $dias = 89;
                                    break;
                                case 3:
                                    $dias = 179;
                                    break;
                            }
                            if ( $diff->days > $dias ) {
                                Clients::where('id', $cliente['id'])->update(["status" => -1]);
                            }
                        } else {
                            $fecha1 = new DateTime(date("Y-m-d", strtotime($cliente['last_charge'])));
                            $fecha2 = new DateTime(date("Y-m-d"));
                            $diff = $fecha1->diff($fecha2);
                            switch ( $cliente['suscription_plan'] ) {
                                case 1: 
                                    $dias = 29;
                                    break;
                                case 2:
                                    $dias = 89;
                                    break;
                                case 3:
                                    $dias = 179;
                                    break;
                            }
                            if ( $diff->days > $dias ) {
                                Clients::where('id', $cliente['id'])->update(["status" => -1]);
                            }
                        }
                    }
                    

                    /**/
                    $usuario_info = [
                        "id" => $cliente->id, 
                        "name" => $cliente->fullname, 
                        "email" => $cliente->email, 
                        "phone" => $cliente->phone, 
                        "country" => $cliente->country, 
                        "nutrition_plan" => $cliente->nutrition_plan, 
                        "nutrition_plan_name" => $Programs['name'], 
                        "workout_plan" => $cliente->workout_plan, 
                        "workout_plan_name" => $Workouts['program'], 
                        "suscription_plan" => $cliente->suscription_plan, 
                        "status" => $cliente->status, 
                        "stripe_id" => $cliente->stripe_source, 
                        "paypal_id" => $cliente->paypal_source
                    ];

                    // Clave secreta para firmar el token (debe mantenerse segura)
                    $clave_secreta = $_ENV['JWT_SECRET'];

                    // Configuración del token
                    $tiempo_de_expiracion = time() + 3600; // El token expira en 1 hora (3600 segundos)

                    // Crear el token JWT
                    $token = JWT::encode($usuario_info, $clave_secreta, 'HS256');
                    /**/
                    
                    $restResponse = 200;
                    $response = array(
                        "error" => false,
                        "message" => "Consulta válida", 
                        "data" => compact('token')
                    );
                } else {
                    $restResponse = 401;
                    $response = array(
                        "error" => true,
                        "message" => "Credenciales invalidas", 
                        "data" => []
                    );
                }
            } catch (JWTException $e) {
                $restResponse = 500;
                $response = array(
                    "error" => true,
                    "message" => "No se pudo crear el token", 
                    "data" => []
                );
            }

            return response()
                ->json($response, $restResponse)
                ->header("Access-Control-Allow-Origin", "*");
        }
    }

    public function token( Request $r ) {
        if ( Request()->header('x-glowme-key') != $_ENV['X_GLOWME_API'] ) { 
            return response()
            ->json([ 
                'error' => true, 
                'message' => "Api key error", 
                'data' => "Acceso no permitido" 
            ], 403)
            ->header("Access-Control-Allow-Origin", "*");
        } else {
            try {
                $cliente = Clients::where('email', $r->email);
                $name = $cliente->first();

                if ($name != null) {
                    list($usec, $sec) = explode(" ", microtime());
                    $token = Hash::make($r->email . date("Y-m-dTH:i:s") . $usec . $sec);
                    $cliente->update(['recovery_token' => $token]);

                    //$view = view('emails/recovery-password',compact( 'token' ))->render();
                    $view = view('emails/recovery-password', ['token' => $token])->render();

                    $mail = new PHPMailer(true);
                    $mail->Host = "smtp.gmail.com";
                    $mail->Port       = 587;
                    $mail->SMTPSecure = '';
                    $mail->SMTPAuth   = true;
                    $mail->IsHTML(true);
                    $mail->CharSet = 'UTF-8';

                    $mail->Username = "hola@glowmeapp.com";
                    $mail->Password = "ckqbyhujrpdesrrk";
                    $mail->SetFrom('hola@glowmeapp.com', 'Hola GlowMe');
                    $mail->addReplyTo('hola@glowmeapp.com', 'Hola GlowMe');

                    $mail->Subject = "GlowMe Recupera tu contraseña";
                    $mail->Body = $view; 
                    $mail->AddAddress($r->email, "{$name->nombre} {$name->apellido}");
                    $mail->addBCC('mauricio.salinas@metodika.mx', 'Leads');
                    $mail->addBCC('leads@metodika.mx', 'Leads');
                    $mail->addBCC('carlos.betancourt@metodika.mx', 'Carlos');
                    $mail->addBCC('clara.viera@metodika.mx', 'Clara');

                    if( $mail->Send() ) {
                        $restResponse = 200;
                        $response = array(
                            "error" => false,
                            "message" => "Envío de correo exitoso", 
                            "data" => []
                        );
                    } else {
                        $restResponse = 500;
                        $response = array(
                            "error" => false,
                            "message" => "Ocurrió un error al enviar el correo", 
                            "data" => []
                        );
                    }
                } else {
                    $restResponse = 401;
                    $response = array(
                        "error" => true,
                        "message" => "La cuenta no existe", 
                        "data" => []
                    );
                }
            } catch (JWTException $e) {
                $restResponse = 500;
                $response = array(
                    "error" => true,
                    "message" => "Ocurrió un error al generar el Token", 
                    "data" => []
                );
            }

            return response()
                ->json($response, $restResponse)
                ->header("Access-Control-Allow-Origin", "*");
        }
    }

    public function repassword( Request $r ) {
        if ( Request()->header('x-glowme-key') != $_ENV['X_GLOWME_API'] ) { 
            return response()
            ->json([ 
                'error' => true, 
                'message' => "Api key error", 
                'data' => "Acceso no permitido" 
            ], 403)
            ->header("Access-Control-Allow-Origin", "*");
        } else {
            try {
                $cliente = Clients::where('recovery_token', $r->token);

                if ($cliente) {
                    $rePass = Hash::make($r->newPass);
                    $cliente->update(['password' => $rePass, 'recovery_token' => '']);

                    $restResponse = 200;
                    $response = array(
                        "error" => false,
                        "message" => "Contraseña guardada", 
                        "data" => []
                    );
                } else {
                    $restResponse = 401;
                    $response = array(
                        "error" => true,
                        "message" => "El token no es valido", 
                        "data" => []
                    );
                }
            } catch (JWTException $e) {
                $restResponse = 500;
                $response = array(
                    "error" => true,
                    "message" => "Ocurrió un error al guardar la contraseña", 
                    "data" => []
                );
            }

            return response()
                ->json($response, $restResponse)
                ->header("Access-Control-Allow-Origin", "*");
        }
    }

    public function saveCustomer( Request $r ) {
        if ( Request()->header('x-glowme-key') != $_ENV['X_GLOWME_API'] ) { 
            return response()
            ->json([ 
                'error' => true, 
                'message' => "Api key error", 
                'data' => "Acceso no permitido" 
            ], 403)
            ->header("Access-Control-Allow-Origin", "*");
        } else {
            try {
                Clients::where('id', $r->cliente)->update([
                    "suscription_id" => $r->subs, 
                    "source_active" => "stripe", 
                    "stripe_source" => $r->customer, 
                    "status" => 1
                ]);
                //Validar el status en stripe

                $restResponse = 200;
                $response = array(
                    "error" => false,
                    "message" => "Customer guardado", 
                    "data" => []
                );
            } catch (JWTException $e) {
                $restResponse = 500;
                $response = array(
                    "error" => true,
                    "message" => "Ocurrió un error al guardar la contraseña", 
                    "data" => []
                );
            }

            return response()
                ->json($response, $restResponse)
                ->header("Access-Control-Allow-Origin", "*");
        }
    }
}