<?php

namespace App\Http\Controllers\Api\Integraciones\Perfil;

use App\Http\Controllers\Controller;
use Illuminate\Http\Request;
use Illuminate\Support\Facades\DB;
use Illuminate\Support\Facades\File;
use Illuminate\Database\QueryException;
use App\Models\Clients;
use App\Models\GeneralData;
use App\Models\CatMainGoal;
use App\Models\CatPhysicalActivity;
use App\Models\CatGoalWeight;
use App\Models\Suscriptions;
use App\Models\CatCancellations;
use App\Models\Cancellations;
use App\Models\HealthyPregnancy;
use App\Models\HealthyLactation;
use App\Models\HealthyPostpartum;
use DateTime;

use PHPMailer\PHPMailer\PHPMailer;
use PHPMailer\PHPMailer\SMTP;
use PHPMailer\PHPMailer\Exception;

class PerfilController extends Controller {

    protected $unidades = array( 1 => "Litros", 2 => "Mililitros", 3 => "Galones", 4 => "Onzas" );
    protected $unidadesPeso = array( 1 => "Kilos", 2 => "Libras" );
    protected $unidadesDistancia = array( 3 => "Centimetros", 4 => "Pulgadas" );
    protected $API_URL = "https://api-m.sandbox.paypal.com/v1";

    private function updateSuscripcionStripe( $subscription, $planActual, $nuevoPlan ) {
        //$Suscriptions = Suscriptions::select("stripe_price")->where("id", $nuevoPlan)->first();

        $stripe = new \Stripe\StripeClient($_ENV['STRIPE_SECRET']);

        $retriveResponse = $stripe->subscriptions->retrieve(
            $subscription,
            []
        );


        $createReponse = $stripe->subscriptions->update(
            $subscription,
            [
                'items' => [
                    [
                        'id' => $retriveResponse['items']['data'][0]['id'],
                        'price' => $nuevoPlan,
                    ],
                ],
            ]
        );
        if ( $createReponse->id ) {
            return array( "response" => "ok", "id" => $createReponse->id );
        } else {
            return array( "response" => "no", "id" => "", "data" => $createReponse );
        }
    }

    private function cancelSuscripcionStripe($subscription) {
        $stripe = new \Stripe\StripeClient($_ENV['STRIPE_SECRET']);

        $createReponse = $stripe->subscriptions->cancel(
          $subscription,
          []
        );
        if ( $createReponse->id ) {
            return array( "response" => "ok", "id" => $createReponse->id );
        } else {
            return array( "response" => "no", "id" => "", "data" => $createReponse );
        }
    }

    private function updateSuscripcionPaypal($paypal_suscription, $plan) {
        // Generar Token
        $headers = array ( 'Accept: application/json', 'Accept-Language: en_US', 'Accept-Language: en_US' );

        $soap_do = curl_init();
        curl_setopt( $soap_do, CURLOPT_URL, $this->API_URL.'/oauth2/token' );
        curl_setopt( $soap_do, CURLOPT_RETURNTRANSFER, true );
        curl_setopt( $soap_do, CURLOPT_POST, 1);
        curl_setopt( $soap_do, CURLOPT_POSTFIELDS, "grant_type=client_credentials");
        curl_setopt( $soap_do, CURLOPT_USERPWD, $_ENV['PAYPAL_USER'] . ':' . $_ENV['PAYPAL_PASS']);

        curl_setopt( $soap_do, CURLOPT_HTTPHEADER, $headers );
        $result = curl_exec($soap_do);
        $Reponse = json_decode($result, true);

        $Token = $Reponse['access_token'];

        // Generated by curl-to-PHP: http://incarnate.github.io/curl-to-php/
        $ch = curl_init();

        $params = [
            "plan_id" =>$plan
        ];

        curl_setopt($ch, CURLOPT_URL, $_ENV['API_PAYPAL_URL'].'/billing/subscriptions/'.$paypal_suscription.'/revise');
        curl_setopt($ch, CURLOPT_RETURNTRANSFER, 1);
        curl_setopt($ch, CURLOPT_POST, 1);
        curl_setopt($ch, CURLOPT_POSTFIELDS, json_encode($params));

        $headers = array();
        $headers[] = 'Content-Type: application/json';
        $headers[] = 'Authorization: Bearer ' . $Token;
        curl_setopt($ch, CURLOPT_HTTPHEADER, $headers);

        $result = curl_exec($ch);

        curl_close($ch);

        if ( curl_errno($ch) ) {
            return array( "response" => "no", "id" => "", "data" => 'Error:' . curl_error($ch) );
        } else {
            return array( "response" => "ok", "id" => $plan );
        }
    }

    private function cancelSuscripcionPaypal($paypal_suscription, $reason) {
        // Generar Token
        $headers = array ( 'Accept: application/json', 'Accept-Language: en_US', 'Accept-Language: en_US' );

        $soap_do = curl_init();
        curl_setopt( $soap_do, CURLOPT_URL, $_ENV['API_PAYPAL_URL'].'/oauth2/token' );
        curl_setopt( $soap_do, CURLOPT_RETURNTRANSFER, true );
        curl_setopt( $soap_do, CURLOPT_POST, 1);
        curl_setopt( $soap_do, CURLOPT_POSTFIELDS, "grant_type=client_credentials");
        curl_setopt( $soap_do, CURLOPT_USERPWD, $_ENV['PAYPAL_USER'] . ':' . $_ENV['PAYPAL_PASS']);

        curl_setopt( $soap_do, CURLOPT_HTTPHEADER, $headers );
        $result = curl_exec($soap_do);
        $Reponse = json_decode($result, true);

        $Token = $Reponse['access_token'];

        // Generated by curl-to-PHP: http://incarnate.github.io/curl-to-php/
        $ch = curl_init();

        curl_setopt($ch, CURLOPT_URL, $_ENV['API_PAYPAL_URL'].'/billing/subscriptions/'.$paypal_suscription.'/cancel');
        curl_setopt($ch, CURLOPT_RETURNTRANSFER, 1);
        curl_setopt($ch, CURLOPT_POST, 1);
        curl_setopt($ch, CURLOPT_POSTFIELDS, "{\n  \"reason\": \"{$reason}\"\n}");

        $headers = array();
        $headers[] = 'Content-Type: application/json';
        $headers[] = 'Accept: application/json';
        $headers[] = 'Authorization: Bearer ' . $Token;
        curl_setopt($ch, CURLOPT_HTTPHEADER, $headers);

        $result = curl_exec($ch);

        if ( curl_errno($ch) ) {
            return array( "response" => "no", "id" => "", "data" => 'Error:' . curl_error($ch) );
        } else {
            return array( "response" => "ok", "id" => $paypal_suscription );
        }
    }

    public function getPerfil( Request $r ) {
        if ( Request()->header('x-glowme-key') != $_ENV['X_GLOWME_API'] ) { 
            return response()
            ->json([ 
                'error' => true, 
                'message' => "Api key error", 
                'data' => "Acceso no permitido" 
            ], 403)
            ->header("Access-Control-Allow-Origin", "*");
        } else {
            try {
                $Clients = Clients::select("id", "fullname", "email", "phone", "country", DB::Raw("CONCAT( '".$_ENV["URL_PATH"]."', image ) as image"))->where("id", $r->id)->first();

                $restResponse = 200;
                $response = array(
                    "error" => false,
                    "message" => "Consulta exitosa", 
                    "data" => $Clients
                );
            } catch (QueryException $e) {
                $restResponse = 500;
                $response = array(
                    "error" => true,
                    "message" => "Error en la consulta", 
                    "data" => $e->getMessage()
                );
            }

            return response()
                ->json($response, $restResponse)
                ->header("Access-Control-Allow-Origin", "*");
        }
    }

    public function getCancelacionesType () {
        if ( Request()->header('x-glowme-key') != $_ENV['X_GLOWME_API'] ) { 
            return response()
            ->json([ 
                'error' => true, 
                'message' => "Api key error", 
                'data' => "Acceso no permitido" 
            ], 403)
            ->header("Access-Control-Allow-Origin", "*");
        } else {
            try {
                $CatCancellations = CatCancellations::select("id", "name")->get()->toArray();

                $restResponse = 200;
                $response = array(
                    "error" => false,
                    "message" => "Consulta exitosa", 
                    "data" => $CatCancellations
                );
            } catch (QueryException $e) {
                $restResponse = 500;
                $response = array(
                    "error" => true,
                    "message" => "Error en la consulta", 
                    "data" => $e->getMessage()
                );
            }

            return response()
                ->json($response, $restResponse)
                ->header("Access-Control-Allow-Origin", "*");
        }
    }

    public function updatePerfil( Request $r ) {
        if ( Request()->header('x-glowme-key') != $_ENV['X_GLOWME_API'] ) { 
            return response()
            ->json([ 
                'error' => true, 
                'message' => "Api key error", 
                'data' => "Acceso no permitido" 
            ], 403)
            ->header("Access-Control-Allow-Origin", "*");
        } else {
            try {

                if ( isset($r->id) && $r->id != "" && $r->id > 0 ) {
                    if ( isset($r->password) && isset($r->fullname) && isset($r->email) && isset($r->phone) && isset($r->country) ) {
                        $updates = [ 
                            "fullname" => $r->fullname, 
                            "email" => $r->email, 
                            "phone" => $r->phone, 
                            "country" => $r->country, 
                            "password" => bcrypt($r->password)
                        ];
                        Clients::where("id", $r->id)->update($updates);
                    } elseif ( isset($r->fullname) && isset($r->email) && isset($r->phone) && isset($r->country) )  {
                        $updates = [
                            "fullname" => $r->fullname, 
                            "email" => $r->email, 
                            "phone" => $r->phone, 
                            "country" => $r->country
                        ];
                        Clients::where("id", $r->id)->update($updates);
                    }            

                    //Files
                    if( $r->has("image") ) {
                        $getImage = Clients::select("image")->where("id", $r->id)->first();
                        if ( $getImage != null ) {
                            $path = ltrim($getImage['image'], '/');
                            if ( File::exists($path) ) {
                                unlink( $path );
                            }
                        }

                        $fName = $r->file('image');
                        $fileName = MD5($fName . uniqid() . microtime()) . "." . $fName->getClientOriginalExtension();
                        $fName->move("img/perfil/{$r->id}", $fileName);
                        $imageUpdate = "/img/perfil/{$r->id}/" . $fileName;

                        Clients::where("id", $r->id)->update(["image" => $imageUpdate]);
                    }

                    $restResponse = 200;
                    $response = array(
                        "error" => false,
                        "message" => "Consulta exitosa", 
                        "data" => []
                    );
                } else {
                    $restResponse = 500;
                    $response = array(
                        "error" => true,
                        "message" => "No se pudo identificar el cliente", 
                        "data" => []
                    );
                }
            } catch (QueryException $e) {
                $restResponse = 500;
                $response = array(
                    "error" => true,
                    "message" => "Error en la consulta", 
                    "data" => $e->getMessage()
                );
            }

            return response()
                ->json($response, $restResponse)
                ->header("Access-Control-Allow-Origin", "*");
        }
    }

    public function getDatosSalud( Request $r ) {
        if ( Request()->header('x-glowme-key') != $_ENV['X_GLOWME_API'] ) { 
            return response()
            ->json([ 
                'error' => true, 
                'message' => "Api key error", 
                'data' => "Acceso no permitido" 
            ], 403)
            ->header("Access-Control-Allow-Origin", "*");
        } else {
            try {
                $Clients = Clients::select("fullname", DB::Raw("CONCAT( '".$_ENV["URL_PATH"]."', image ) as image"))->where("id", $r->id)->first();
                $GeneralData = GeneralData::select(
                    "birth_date", 
                    "weight", 
                    "weight_unit", 
                    "height", 
                    "height_unit", 
                    "waist", 
                    "waist_unit", 
                    "hip",
                    "hip_unit", 
                    "main_goal", 
                    "weight_goal",
                    "physical_activity", 
                    "exceptions", 
                    "hormonal_disorders"
                )->where("i_client", $r->id)->first();

                $CatMainGoal = CatMainGoal::select("goal")->where("id", $GeneralData['main_goal'])->first();
                $CatGoalWeight = CatGoalWeight::select("goal")->where("id", $GeneralData['weight_goal'])->first();

                $CatPhysicalActivity = CatPhysicalActivity::select("level")->where("id", $GeneralData['physical_activity'])->first();

                $nacimiento = new DateTime($GeneralData['birth_date']);
                $ahora = new DateTime(date("Y-m-d"));
                $diferencia = $ahora->diff($nacimiento);
                $edad = $diferencia->format("%y");

                $HealthyPregnancy = HealthyPregnancy::where("i_client", $r->id)->first();
                $HealthyLactation = HealthyLactation::where("i_client", $r->id)->first();
                $HealthyPostpartum = HealthyPostpartum::where("i_client", $r->id)->first();

                if ( $HealthyPregnancy != null ) {
                    $caso = "pregnancy";
                    $casoArray = array(
                        "weeks" => $HealthyPregnancy['weeks'], 
                        "weight" => $HealthyPregnancy['weight'], 
                        "weight_unit" => $HealthyPregnancy['weight_unit'], 
                        "babies" => $HealthyPregnancy['babys']
                    );
                } elseif ( $HealthyLactation != null ) {
                    $caso = "healthy_lactation";
                    $casoArray = array(
                        "weeks" => $HealthyLactation['weeks'], 
                        "weight" => $HealthyLactation['weight'], 
                        "weight_unit" => $HealthyLactation['weight_unit'], 
                        "lactation" => $HealthyLactation['lactation']
                    );
                } elseif ( $HealthyPostpartum != null ) {
                    $caso = "postpartum";
                    $casoArray = array(
                        "weeks" => $HealthyPostpartum['weeks'], 
                        "weight" => $HealthyPostpartum['weight'], 
                        "weight_unit" => $HealthyPostpartum['weight_unit'], 
                        "lactation" => $HealthyPostpartum['lactation']
                    );
                }

                if ( isset($caso) ) {
                    $result = [
                        "nombre" => $Clients["fullname"], 
                        "image" => $Clients["image"],
                        "edad" => $edad,
                        "nacimiento" => $GeneralData['birth_date'],
                        "peso" => number_format($GeneralData['weight'], '0'),
                        "peso_unidad" => $this->unidadesPeso[$GeneralData['weight_unit']],
                        "altura" => $GeneralData['height'] == null ? 0 : number_format($GeneralData['height'], '0'),
                        "altura_unidad" => $GeneralData['height_unit'] = null ? 1 : $this->unidadesDistancia[$GeneralData['height_unit']],
                        "cintura" => $GeneralData['waist'] == null ? 0 : number_format($GeneralData['waist'], '0'),
                        "cintura_unidad" => $GeneralData['waist_unit'] == null ? 1 : $this->unidadesDistancia[$GeneralData['waist_unit']],
                        "cadera" => $GeneralData['hip'] = null ? 0 : number_format($GeneralData['hip'], '0'),
                        "cadera_unidad" => $GeneralData['hip_unit'] == null ? 1 : $this->unidadesDistancia[$GeneralData['hip_unit']],
                        "objetivo" => $GeneralData['main_goal'], 
                        "objetivo_nombre" => $CatMainGoal['goal'], 
                        "objetivo_peso" => $GeneralData['weight_goal'], 
                        "objetivo_peso_nombre" => isset($CatGoalWeight['goal']) ? $CatGoalWeight['goal'] : "", 
                        "actividad_fisica" => $CatPhysicalActivity['level'], 
                        "exceptions" => [ 
                            "total" => $GeneralData['exceptions'] == null ? 0 : count(json_decode($GeneralData['exceptions'])), 
                            "items" => $GeneralData['exceptions'] == null ? [] : json_decode($GeneralData['exceptions']) 
                        ],
                        "hormonal" => [ 
                            "total" => $GeneralData['hormonal_disorders'] == null ? 0 : count(json_decode($GeneralData['hormonal_disorders'])), 
                            "items" => $GeneralData['hormonal_disorders'] == null ? [] : json_decode($GeneralData['hormonal_disorders'])
                        ],
                        $caso => $casoArray
                    ];
                } else {
                    $result = [
                        "nombre" => $Clients["fullname"], 
                        "image" => $Clients["image"],
                        "edad" => $edad,
                        "nacimiento" => $GeneralData['birth_date'],
                        "peso" => number_format($GeneralData['weight'], '0'),
                        "peso_unidad" => $this->unidadesPeso[$GeneralData['weight_unit']],
                        "altura" => $GeneralData['height'] == null ? 0 : number_format($GeneralData['height'], '0'),
                        "altura_unidad" => $GeneralData['height_unit'] = null ? 1 : $this->unidadesDistancia[$GeneralData['height_unit']],
                        "cintura" => $GeneralData['waist'] == null ? 0 : number_format($GeneralData['waist'], '0'),
                        "cintura_unidad" => $GeneralData['waist_unit'] == null ? 1 : $this->unidadesDistancia[$GeneralData['waist_unit']],
                        "cadera" => $GeneralData['hip'] = null ? 0 : number_format($GeneralData['hip'], '0'),
                        "cadera_unidad" => $GeneralData['hip_unit'] == null ? 1 : $this->unidadesDistancia[$GeneralData['hip_unit']],
                        "objetivo" => $GeneralData['main_goal'], 
                        "objetivo_nombre" => $CatMainGoal['goal'], 
                        "objetivo_peso" => $GeneralData['weight_goal'], 
                        "objetivo_peso_nombre" => isset($CatGoalWeight['goal']) ? $CatGoalWeight['goal'] : "", 
                        "actividad_fisica" => $CatPhysicalActivity['level'], 
                        "exceptions" => [ 
                            "total" => $GeneralData['exceptions'] == null ? 0 : count(json_decode($GeneralData['exceptions'])), 
                            "items" => $GeneralData['exceptions'] == null ? [] : json_decode($GeneralData['exceptions']) 
                        ],
                        "hormonal" => [ 
                            "total" => $GeneralData['hormonal_disorders'] == null ? 0 : count(json_decode($GeneralData['hormonal_disorders'])), 
                            "items" => $GeneralData['hormonal_disorders'] == null ? [] : json_decode($GeneralData['hormonal_disorders'])
                        ]
                    ];
                }

                

                $restResponse = 200;
                $response = array(
                    "error" => false,
                    "message" => "Consulta exitosa", 
                    "data" => $result
                );
            } catch (QueryException $e) {
                $restResponse = 500;
                $response = array(
                    "error" => true,
                    "message" => "Error en la consulta", 
                    "data" => $e->getMessage()
                );
            }

            return response()
                ->json($response, $restResponse)
                ->header("Access-Control-Allow-Origin", "*");
        }
    }

    public function updateDatosSalud( Request $r ) {
        if ( Request()->header('x-glowme-key') != $_ENV['X_GLOWME_API'] ) { 
            return response()
            ->json([ 
                'error' => true, 
                'message' => "Api key error", 
                'data' => "Acceso no permitido" 
            ], 403)
            ->header("Access-Control-Allow-Origin", "*");
        } else {
            try {
                GeneralData::where("i_client", $r->client)->update([
                    "birth_date" => $r->birth,
                    "weight" => $r->weight,
                    "weight_unit" => $r->weight_unit,
                    "height" => $r->height,
                    "height_unit" => $r->height_unit,
                    "waist" => $r->waist,
                    "waist_unit" => $r->waist_unit,
                    "hip" => $r->hip,
                    "hip_unit" => $r->hip_unit,
                    "main_goal" => $r->main_goal,
                    "weight_goal" => $r->weight_goal,
                    "physical_activity" => $r->physical_activity,
                    "exceptions" => json_encode($r->exceptions),
                    "hormonal_disorders" => json_encode($r->hormonal_disorders)
                ]);

                HealthyPregnancy::where("i_client", $r->client)->delete();
                HealthyLactation::where("i_client", $r->client)->delete();
                HealthyPostpartum::where("i_client", $r->client)->delete();

                if ( $r->has("pregnancy") ) {
                    $HealthyPregnancy = HealthyPregnancy::updateOrCreate(
                        ['i_client' => $r->client], 
                        [
                            'weeks' => $r->pregnancy['weeks'], 
                            'weight' => $r->pregnancy['weight'], 
                            'weight_unit' => $r->pregnancy['weight_unit'], 
                            'babys' => $r->pregnancy['babies'], 
                        ]
                    );
                }

                if ( $r->has("healthy_lactation") ) {
                    $HealthyLactation = HealthyLactation::updateOrCreate(
                        ['i_client' => $r->client], 
                        [
                            'weeks' => $r->healthy_lactation['weeks'], 
                            'weight' => $r->healthy_lactation['weight'], 
                            'weight_unit' => $r->healthy_lactation['weight_unit'], 
                            'lactation' => $r->healthy_lactation['lactation'], 
                        ]
                    );
                }

                if ( $r->has("postpartum") ) {
                    $HealthyPostpartum = HealthyPostpartum::updateOrCreate(
                        ['i_client' => $r->client], 
                        [
                            'weeks' => $r->postpartum['weeks'], 
                            'weight' => $r->postpartum['weight'], 
                            'weight_unit' => $r->postpartum['weight_unit'], 
                            'lactation' => $r->postpartum['lactation'], 
                        ]
                    );
                }

                $restResponse = 200;
                $response = array(
                    "error" => false,
                    "message" => "Consulta exitosa", 
                    "data" => []
                );
            } catch (QueryException $e) {
                $restResponse = 500;
                $response = array(
                    "error" => true,
                    "message" => "Error en la consulta", 
                    "data" => $e->getMessage()
                );
            }

            return response()
                ->json($response, $restResponse)
                ->header("Access-Control-Allow-Origin", "*");
        }
    }

    public function getSuscripcion( Request $r ) {
        if ( Request()->header('x-glowme-key') != $_ENV['X_GLOWME_API'] ) { 
            return response()
            ->json([ 
                'error' => true, 
                'message' => "Api key error", 
                'data' => "Acceso no permitido" 
            ], 403)
            ->header("Access-Control-Allow-Origin", "*");
        } else {
            try {
                $Clients = Clients::select("fullname", "suscription_plan", "suscription_id", "source_active", "paypal_source")->where("id", $r->id)->first();

                if ( $Clients['source_active'] == 'stripe') {
                    $stripe = new \Stripe\StripeClient($_ENV['STRIPE_SECRET']);

                    $createReponse = $stripe->subscriptions->retrieve(
                        $Clients['suscription_id'],
                        []
                    );
                } else {
                    // Generar Token
                    $params = '{
                        "grant_type": "client_credentials"
                    }';

                    $headers = array ( 'Accept: application/json', 'Accept-Language: en_US', 'Accept-Language: en_US' );

                    $soap_do = curl_init();
                    curl_setopt( $soap_do, CURLOPT_URL, $_ENV['API_PAYPAL_URL'].'/oauth2/token' );
                    curl_setopt( $soap_do, CURLOPT_RETURNTRANSFER, true );
                    curl_setopt( $soap_do, CURLOPT_POST, 1);
                    curl_setopt( $soap_do, CURLOPT_POSTFIELDS, "grant_type=client_credentials");
                    curl_setopt( $soap_do, CURLOPT_USERPWD, 'Ac6hbSHwiUwMXgbhn0YrBtzQ0EsfBBtRu4zwmLG9cP-g-K7mhFVGVkZvSytiE0UwahDVTHiuIrHQoY2E' . ':' . 'EDMfJryobI5gqUVb5xw7D7lUH-_Yj0O9MXxlChe15NtyVP5GkK74GdW8sjWC92gn7sR79lpE9w2fIqa5');

                    curl_setopt( $soap_do, CURLOPT_HTTPHEADER, $headers );
                    $result = curl_exec($soap_do);
                    $Reponse = json_decode($result, true);

                    $Token = $Reponse['access_token'];

                    //

                    $ch = curl_init();

                    curl_setopt($ch, CURLOPT_URL, $_ENV['API_PAYPAL_URL'].'/billing/subscriptions/'.$Clients['paypal_source']);
                    curl_setopt($ch, CURLOPT_RETURNTRANSFER, 1);
                    curl_setopt($ch, CURLOPT_CUSTOMREQUEST, 'GET');


                    $headers = array ( 'Authorization: Bearer ' . $Token, 'Content-Type: application/json' );
                    $headers[] = 'Content-Type: application/json';
                    $headers[] = 'Accept: application/json';
                    curl_setopt($ch, CURLOPT_HTTPHEADER, $headers);
                    $result = curl_exec($ch);
                    $result = json_decode($result, true);

                    $createReponse['current_period_end'] = isset($result['billing_info']['next_billing_time']) ? strtotime($result['billing_info']['next_billing_time']) : "0";

                    curl_close($ch);
                }

                $Suscriptions = Suscriptions::select(
                    "id", 
                    "name", 
                    "desc", 
                    "cost", 
                    "plan"
                )->where("id", $Clients['suscription_plan'])
                ->first();

                $result = [
                    "nombre" => $Clients["fullname"], 
                    "id_plan" => $Suscriptions["id"],
                    "nombre_plan" => $Suscriptions["name"],
                    "desc" => $Suscriptions['desc'],
                    "costo" => $Suscriptions['cost'],
                    "plan" => $Suscriptions['plan'], 
                    "vencimiento" => date("d / m / Y", $createReponse['current_period_end'])
                ];

                $restResponse = 200;
                $response = array(
                    "error" => false,
                    "message" => "Consulta exitosa", 
                    "data" => $result
                );
            } catch (QueryException $e) {
                $restResponse = 500;
                $response = array(
                    "error" => true,
                    "message" => "Error en la consulta", 
                    "data" => $e->getMessage()
                );
            }

            return response()
                ->json($response, $restResponse)
                ->header("Access-Control-Allow-Origin", "*");
        }    
    }

    public function updateSuscripcion( Request $r ) {
        if ( Request()->header('x-glowme-key') != $_ENV['X_GLOWME_API'] ) { 
            return response()
            ->json([ 
                'error' => true, 
                'message' => "Api key error", 
                'data' => "Acceso no permitido" 
            ], 403)
            ->header("Access-Control-Allow-Origin", "*");
        } else {
            try {               
                $Clients = Clients::select("paypal_source", "source_active", "suscription_plan", "suscription_id")->where("id", $r->client)->first();

                $planDB = 0;
                switch( $Clients['source_active'] ) {
                    case "stripe":
                        $result = $this->updateSuscripcionStripe($Clients['suscription_id'], $Clients['suscription_plan'], $r->plan);
                        if ( $result['response'] == "ok" ) {
                            $sus = Suscriptions::select("id")->where("stripe_price", $r->plan)->first();
                            $planDB = $sus['id'];
                            Clients::where("id", $r->client)->update(["suscription_plan" => $sus['id'], "suscription_id" => $result['id']]);
                            $response = array(
                                "error" => false,
                                "message" => "Consulta exitosa", 
                                "data" => []
                            );
                        } else {
                            $response = array(
                                "error" => true,
                                "message" => "Error en la actualización", 
                                "data" => $result['data']
                            );
                        }
                        break;
                    case "paypal": 
                        $result = $this->updateSuscripcionPaypal($Clients['paypal_source'], $r->plan);
                        if ( $result['response'] == "ok" ) {
                            $sus = Suscriptions::select("id")->where("paypal_plan", $r->plan)->first();
                            $planDB = $sus['id'];
                            Clients::where("id", $r->client)->update(["suscription_id" => "", "suscription_plan" => $sus['id']]);

                            $response = array(
                                "error" => false,
                                "message" => "Consulta exitosa", 
                                "data" => []
                            );
                        } else {
                            $response = array(
                                "error" => true,
                                "message" => "Error en la actualización", 
                                "data" => $result['data']
                            );
                        }
                        break;
                }

                /***/
                /***/
                    $data = Clients::select("fullname", "email", "suscription_plan", "suscription_id", "source_active", "paypal_source")->where("id", $r->client)->first();
                    $old = $data['suscription_plan'];
                    $new = $planDB;

                    $oldData = Suscriptions::select("name", "cost", DB::Raw("
                        CASE
                            WHEN id = 1 THEN cost/1
                            WHEN id = 2 THEN cost/3
                            WHEN id = 3 THEN cost/6
                        END
                    as plan"))->where("id", $old)->first();
                    $newData = Suscriptions::select("name", "cost", DB::Raw("
                        CASE
                            WHEN id = 1 THEN cost/1
                            WHEN id = 2 THEN cost/3
                            WHEN id = 3 THEN cost/6
                        END
                    as plan"))->where("id", $new)->first();


                    /***/
                    if ( $data['source_active'] == 'stripe') {
                        $stripe = new \Stripe\StripeClient($_ENV['STRIPE_SECRET']);

                        $createReponse = $stripe->subscriptions->retrieve(
                            $data['suscription_id'],
                            []
                        );
                    } else {
                        // Generar Token
                        $params = '{
                            "grant_type": "client_credentials"
                        }';

                        $headers = array ( 'Accept: application/json', 'Accept-Language: en_US', 'Accept-Language: en_US' );

                        $soap_do = curl_init();
                        curl_setopt( $soap_do, CURLOPT_URL, $_ENV['API_PAYPAL_URL'].'/oauth2/token' );
                        curl_setopt( $soap_do, CURLOPT_RETURNTRANSFER, true );
                        curl_setopt( $soap_do, CURLOPT_POST, 1);
                        curl_setopt( $soap_do, CURLOPT_POSTFIELDS, "grant_type=client_credentials");
                        curl_setopt( $soap_do, CURLOPT_USERPWD, 'Ac6hbSHwiUwMXgbhn0YrBtzQ0EsfBBtRu4zwmLG9cP-g-K7mhFVGVkZvSytiE0UwahDVTHiuIrHQoY2E' . ':' . 'EDMfJryobI5gqUVb5xw7D7lUH-_Yj0O9MXxlChe15NtyVP5GkK74GdW8sjWC92gn7sR79lpE9w2fIqa5');

                        curl_setopt( $soap_do, CURLOPT_HTTPHEADER, $headers );
                        $result = curl_exec($soap_do);
                        $Reponse = json_decode($result, true);

                        $Token = $Reponse['access_token'];

                        //

                        $ch = curl_init();

                        curl_setopt($ch, CURLOPT_URL, $_ENV['API_PAYPAL_URL'].'/billing/subscriptions/'.$data['paypal_source']);
                        curl_setopt($ch, CURLOPT_RETURNTRANSFER, 1);
                        curl_setopt($ch, CURLOPT_CUSTOMREQUEST, 'GET');


                        $headers = array ( 'Authorization: Bearer ' . $Token, 'Content-Type: application/json' );
                        $headers[] = 'Content-Type: application/json';
                        $headers[] = 'Accept: application/json';
                        curl_setopt($ch, CURLOPT_HTTPHEADER, $headers);
                        $result = curl_exec($ch);
                        $result = json_decode($result, true);

                        $createReponse['current_period_end'] = strtotime($result['billing_info']['next_billing_time']);

                        curl_close($ch);
                    }
                    /***/

                    $view = view('emails/cambio-plan', [ 'name' => $data['fullname'], 'oldData' => $oldData, 'newData' => $newData, 'next' => date("d / m / Y", $createReponse['current_period_end']) ])->render();

                    $mail = new PHPMailer(true);
                    $mail->Host = "smtp.gmail.com";
                    $mail->Port       = 587;
                    $mail->SMTPSecure = '';
                    $mail->SMTPAuth   = true;
                    $mail->IsHTML(true);
                    $mail->CharSet = 'UTF-8';

                    $mail->Username = "hola@glowmeapp.com";
                    $mail->Password = "ckqbyhujrpdesrrk";
                    $mail->SetFrom('hola@glowmeapp.com', 'Hola GlowMe');
                    $mail->addReplyTo('hola@glowmeapp.com', 'Hola GlowMe');

                    $mail->Subject = "GlowMe Cambio de Plan";
                    $mail->Body = $view; 
                    $mail->AddAddress($data['email'], "{$data['fullname']}");
                    $mail->addBCC('mauricio.salinas@metodika.mx', 'Leads');
                    $mail->addBCC('leads@metodika.mx', 'Leads');
                    $mail->addBCC('carlos.betancourt@metodika.mx', 'Carlos');
                    $mail->addBCC('clara.viera@metodika.mx', 'Clara');

                    $mail->Send(); 
                /***/
                /***/

                $restResponse = 200;               
            } catch (QueryException $e) {
                $restResponse = 500;
                $response = array(
                    "error" => true,
                    "message" => "Error en la consulta", 
                    "data" => $e->getMessage()
                );
            }

            return response()
                ->json($response, $restResponse)
                ->header("Access-Control-Allow-Origin", "*");
        }            
    }

    public function cancelSuscripcion( Request $r ) {
        if ( Request()->header('x-glowme-key') != $_ENV['X_GLOWME_API'] ) { 
            return response()
            ->json([ 
                'error' => true, 
                'message' => "Api key error", 
                'data' => "Acceso no permitido" 
            ], 403)
            ->header("Access-Control-Allow-Origin", "*");
        } else {
            try {

                $Clients = Clients::select("fullname", "email", "paypal_source", "source_active", "suscription_id")->where("id", $r->client)->first();

                //Clients::where("id", $r->client)->update(["status" => -1]);

                switch( $Clients['source_active'] ) {
                    case "stripe":
                        $result = $this->cancelSuscripcionStripe($Clients['suscription_id']);
                        if ( $result['response'] == "ok" ) {
                            Clients::where("id", $r->client)->update(["suscription_cancel_at" => date("Y-m-d H:i:s")]);
                            
                            $Cancellations = new Cancellations();
                            $Cancellations->i_client = $r->client;
                            $Cancellations->i_cancellation = $r->reason_id;
                            $Cancellations->tx_reason = $r->reason;
                            $Cancellations->save();

                            $response = array(
                                "error" => false,
                                "message" => "Consulta exitosa", 
                                "data" => []
                            );
                        } else {
                            $response = array(
                                "error" => true,
                                "message" => "Error en la actualización", 
                                "data" => $result['data']
                            );
                        }
                        break;
                    case "paypal": 
                        $result = $this->cancelSuscripcionPaypal($Clients['paypal_source'], $r->reason);

                        if ( $result['response'] == "ok" ) {
                            Clients::where("id", $r->client)->update(["suscription_cancel_at" => date("Y-m-d H:i:s")]);
                            
                            $Cancellations = new Cancellations();
                            $Cancellations->i_client = $r->client;
                            $Cancellations->i_cancellation = $r->reason_id;
                            $Cancellations->tx_reason = $r->reason;
                            $Cancellations->save();

                            $response = array(
                                "error" => false,
                                "message" => "Consulta exitosa", 
                                "data" => []
                            );
                        } else {
                            $response = array(
                                "error" => true,
                                "message" => "Error en la actualización", 
                                "data" => $result['data']
                            );
                        }
                        break;
                }

                $view = view('emails/cancelacion', ['name' => $Clients['fullname']])->render();

                $mail = new PHPMailer(true);
                $mail->Host = "smtp.gmail.com";
                $mail->Port       = 587;
                $mail->SMTPSecure = '';
                $mail->SMTPAuth   = true;
                $mail->IsHTML(true);
                $mail->CharSet = 'UTF-8';

                $mail->Username = "hola@glowmeapp.com";
                $mail->Password = "ckqbyhujrpdesrrk";
                $mail->SetFrom('hola@glowmeapp.com', 'Hola GlowMe');
                $mail->addReplyTo('hola@glowmeapp.com', 'Hola GlowMe');
                
                $mail->Subject = "GlowMe Cancelación";
                $mail->Body = $view;
                $mail->AddAddress($Clients['email'], "{$Clients['fullname']}");
                $mail->addBCC('mauricio.salinas@metodika.mx', 'Leads');
                $mail->addBCC('leads@metodika.mx', 'Leads');
                $mail->addBCC('carlos.betancourt@metodika.mx', 'Carlos');
                $mail->addBCC('clara.viera@metodika.mx', 'Clara');

                $mail->Send();

                $restResponse = 200;
                
            } catch (QueryException $e) {
                $restResponse = 500;
                $response = array(
                    "error" => true,
                    "message" => "Error en la consulta", 
                    "data" => $e->getMessage()
                );
            }

            return response()
                ->json($response, $restResponse)
                ->header("Access-Control-Allow-Origin", "*");
        }            
    }

    public function updatePregnancy ( Request $r ) {
        if ( Request()->header('x-glowme-key') != $_ENV['X_GLOWME_API'] ) { 
            return response()
            ->json([ 
                'error' => true, 
                'message' => "Api key error", 
                'data' => "Acceso no permitido" 
            ], 403)
            ->header("Access-Control-Allow-Origin", "*");
        } else {
            try {
                $restResponse = 200;

                HealthyPregnancy::where("i_client", $r->client)->delete();
                HealthyLactation::where("i_client", $r->client)->delete();
                HealthyPostpartum::where("i_client", $r->client)->delete();

                if ( $r->has("pregnancy") ) {
                    $HealthyPregnancy = HealthyPregnancy::updateOrCreate(
                        ['i_client' => $r->client], 
                        [
                            'weeks' => $r->pregnancy['weeks'], 
                            'weight' => $r->pregnancy['weight'], 
                            'weight_unit' => $r->pregnancy['weight_unit'], 
                            'babys' => $r->pregnancy['babies'], 
                        ]
                    );
                }

                if ( $r->has("healthy_lactation") ) {
                    $HealthyLactation = HealthyLactation::updateOrCreate(
                        ['i_client' => $r->client], 
                        [
                            'weeks' => $r->healthy_lactation['weeks'], 
                            'weight' => $r->healthy_lactation['weight'], 
                            'weight_unit' => $r->healthy_lactation['weight_unit'], 
                            'lactation' => $r->healthy_lactation['lactation'], 
                        ]
                    );
                }

                if ( $r->has("postpartum") ) {
                    $HealthyPostpartum = HealthyPostpartum::updateOrCreate(
                        ['i_client' => $r->client], 
                        [
                            'weeks' => $r->postpartum['weeks'], 
                            'weight' => $r->postpartum['weight'], 
                            'weight_unit' => $r->postpartum['weight_unit'], 
                            'lactation' => $r->postpartum['lactation'], 
                        ]
                    );
                }

                $restResponse = 200;
                $response = array(
                    "error" => false,
                    "message" => "Consulta exitosa", 
                    "data" => []
                );
            } catch (QueryException $e) {
                $restResponse = 500;
                $response = array(
                    "error" => true,
                    "message" => "Error en la consulta", 
                    "data" => $e->getMessage()
                );
            }

            return response()
                ->json($response, $restResponse)
                ->header("Access-Control-Allow-Origin", "*");
        }
    }
}