<?php

namespace App\Http\Controllers\Api\Learn;

use App\Http\Controllers\Controller;
use Illuminate\Http\Request;
use Illuminate\Support\Facades\DB;
use Illuminate\Database\QueryException;
use App\Models\LearnCategories;
use App\Models\LearnVideos;
use Illuminate\Support\Facades\File;

class LearnController extends Controller {
    #Categorias
    public function getCategorias() {
        $response = array(
            "error" => false,
            "message" => "Sin error"
        );

        try {
            $learnCategories = LearnCategories::select(
                "id", 
                "name"
            )
            ->get();

            $response = array(
                "error" => true,
                "message" => "Sin error", 
                "data" => $learnCategories
            );
        } catch ( QueryException $e ) {
            $response = array(
                "error" => true,
                "message" => "Error en la consulta", 
                "data" => $e->getMessage()
            );
        }

        return response()
            ->json($response, 200)
            ->header("Access-Control-Allow-Origin", "*");
    }

    public function getCategoriaID($id) {
    	$response = array(
            "error" => false,
            "message" => "Sin error"
        );

        try {
            $LearnCategories = LearnCategories::select(
        		"name", 
                "image"
        	)
            ->where("id", $id)
            ->get();

			$response = array(
                "error" => false,
                "message" => "Sin error", 
                "data" => $LearnCategories
            );

        } catch ( QueryException $e ) {
            $response = array(
                "error" => true,
                "message" => "Error en la consulta", 
                "data" => $e->getMessage()
            );
        }

        return response()
            ->json($response, 200)
            ->header("Access-Control-Allow-Origin", "*");
    }

    public function storeCategorias(Request $r) {
        $response = array(
            "error" => false,
            "message" => "Sin error"
        );

        $response = array(
            "error" => true,
            "message" => "Error en la consulta", 
            "data" => $r->post()
        );

        try {
        	$LearnCategories = new LearnCategories();
        	$LearnCategories->name = $r->name;

            //Files
            if( $r->has("image") ) {
                $fName = $r->file('image');
                $fileName = MD5($fName . uniqid() . microtime()) . "." . $fName->getClientOriginalExtension();
                $r->file('image')->move("img/categorias/image", $fileName);
                $LearnCategories->image = "/img/categorias/image/" . $fileName;
            }

			if ( $LearnCategories->save() ) {
	            $response = array(
	                "error" => false,
	                "message" => "Sin error", 
	                "data" => []
	            );
	        } else {
	        	$response = array(
	                "error" => true,
	                "message" => "Error en la consulta", 
	                "data" => $e->getMessage()
	            );
	        }
        } catch ( QueryException $e ) {
            $response = array(
                "error" => true,
                "message" => "Error en la consulta", 
                "data" => $e->getMessage()
            );
        }

        return response()
            ->json($response, 200)
            ->header("Access-Control-Allow-Origin", "*");
    }

    public function updateCategorias(Request $r) {
        $response = array(
            "error" => false,
            "message" => "Sin error"
        );

        try {
        	$LearnCategories = LearnCategories::find($r->id);
            $LearnCategories->name = $r->name;

            //Files
            if( $r->has("image") ) {
                //Borrar Banner
                $path = ltrim($LearnCategories->image, '/');
                if ( File::exists($path) ) {
                    unlink( $path );
                }

                $fName = $r->file('image');
                $fileName = MD5($fName . uniqid() . microtime()) . "." . $fName->getClientOriginalExtension();
                $r->file('image')->move("img/categorias/image", $fileName);
                $LearnCategories->image = "/img/categorias/image/" . $fileName;
            } 

			if ( $LearnCategories->save() ) {
	            $response = array(
	                "error" => false,
	                "message" => "Sin error", 
	                "data" => $LearnCategories
	            );
	        } else {
	        	$response = array(
	                "error" => true,
	                "message" => "Error en la consulta", 
	                "data" => $e->getMessage()
	            );
	        }
        } catch ( QueryException $e ) {
            $response = array(
                "error" => true,
                "message" => "Error en la consulta", 
                "data" => $e->getMessage()
            );
        }

        return response()
            ->json($response, 200)
            ->header("Access-Control-Allow-Origin", "*");
    }

    public function deleteCategorias(Request $r) {
        $response = array(
            "error" => false,
            "message" => "Sin error"
        );

        try {
        	$LearnCategories = LearnCategories::find($r->id);

            $videos = LearnVideos::where("i_category", $r->id)->get()->toArray();

            if ( $videos == null ) {
    			if ( $LearnCategories->delete() ) {
                    //Borrar image
                    $path = ltrim($LearnCategories->image, '/');
                    if ( File::exists($path) ) {
                        unlink( $path );
                    }
                    
    	            $response = array(
    	                "error" => false,
    	                "message" => "Sin error", 
    	                "data" => $LearnCategories
    	            );
    	        } else {
    	        	$response = array(
    	                "error" => true,
    	                "message" => "Error en la consulta", 
    	                "data" => $e->getMessage()
    	            );
    	        }
            } else {
                $response = array(
                    "error" => true,
                    "message" => "Hay videos asignados a esta categoría, no es posible eliminarlas", 
                    "data" => []
                );
            }
        } catch ( QueryException $e ) {
            $response = array(
                "error" => true,
                "message" => "Error en la consulta", 
                "data" => $e->getMessage()
            );
        }

        return response()
            ->json($response, 200)
            ->header("Access-Control-Allow-Origin", "*");
    }


    #Videos
    public function getVideos() {
        $response = array(
            "error" => false,
            "message" => "Sin error"
        );

        try {
            $LearnVideos = LearnVideos::select(
                "id", 
                "i_category",
                "title",
                "desc",
                "preview"
            )
            ->get();

            $result = array();
            foreach ( $LearnVideos as $item ) {
                $cats = LearnCategories::select("name")->where("id", $item['i_category'])->first()->toArray();

                $videos = array(
                    "id" => $item['id'], 
                    "i_category" => $item['i_category'],
                    "name" => $cats['name'], 
                    "title" => $item['title'],
                    "desc" => $item['desc'],
                );
                array_push($result, $videos);

            }

            $response = array(
                "error" => true,
                "message" => "Sin error", 
                "data" => $result
            );
        } catch ( QueryException $e ) {
            $response = array(
                "error" => true,
                "message" => "Error en la consulta", 
                "data" => $e->getMessage()
            );
        }

        return response()
            ->json($response, 200)
            ->header("Access-Control-Allow-Origin", "*");
    }

    public function getVideoID($id) {
        $response = array(
            "error" => false,
            "message" => "Sin error"
        );

        try {
            $LearnVideos = LearnVideos::select(
                "id", 
                "i_category",
                "title",
                "desc",
                "vimeo", 
                "preview", 
                "videoWeb", 
                "videoMobile"
            )
            ->where("id", $id)
            ->get();

            $result = array();
            foreach ( $LearnVideos as $item ) {
                $cats = LearnCategories::select("name")->where("id", $item['i_category'])->first()->toArray();

                $videos = array(
                    "id" => $item['id'], 
                    "i_category" => $item['i_category'],
                    "name" => $cats['name'], 
                    "title" => $item['title'],
                    "desc" => $item['desc'],
                    "vimeo" => $item['vimeo'], 
                    "preview" => $item['preview'], 
                    "videoWeb" => $item['videoWeb'], 
                    "videoMobile" => $item['videoMobile']
                );
                array_push($result, $videos);

            }

            $response = array(
                "error" => false,
                "message" => "Sin error", 
                "data" => $result
            );

        } catch ( QueryException $e ) {
            $response = array(
                "error" => true,
                "message" => "Error en la consulta", 
                "data" => $e->getMessage()
            );
        }

        return response()
            ->json($response, 200)
            ->header("Access-Control-Allow-Origin", "*");
    }

    public function storVideos(Request $r) {
        $response = array(
            "error" => false,
            "message" => "Sin error"
        );

        $response = array(
            "error" => true,
            "message" => "Error en la consulta", 
            "data" => $r->post()
        );

        try {
            $LearnVideos = new LearnVideos();
            $LearnVideos->i_category = $r->categoria;
            $LearnVideos->title = $r->name;
            $LearnVideos->desc = $r->desc;
            $LearnVideos->vimeo = $r->liga;
            $LearnVideos->preview = $r->preview;
            $LearnVideos->videoWeb = $r->videoWeb;
            $LearnVideos->videoMobile = $r->videoMobile;

            if ( $LearnVideos->save() ) {
                $response = array(
                    "error" => false,
                    "message" => "Sin error", 
                    "data" => []
                );
            } else {
                $response = array(
                    "error" => true,
                    "message" => "Error en la consulta", 
                    "data" => $e->getMessage()
                );
            }
        } catch ( QueryException $e ) {
            $response = array(
                "error" => true,
                "message" => "Error en la consulta", 
                "data" => $e->getMessage()
            );
        }

        return response()
            ->json($response, 200)
            ->header("Access-Control-Allow-Origin", "*");
    }

    public function updateVideos(Request $r) {
        $response = array(
            "error" => false,
            "message" => "Sin error"
        );

        try {
            $LearnVideos = LearnVideos::find($r->id);
            $LearnVideos->i_category = $r->categoria;
            $LearnVideos->title = $r->name;
            $LearnVideos->desc = $r->desc;
            $LearnVideos->vimeo = $r->liga;
            $LearnVideos->preview = $r->preview;
            $LearnVideos->videoWeb = $r->videoWeb;
            $LearnVideos->videoMobile = $r->videoMobile;

            if ( $LearnVideos->save() ) {
                $response = array(
                    "error" => false,
                    "message" => "Sin error", 
                    "data" => $LearnVideos
                );
            } else {
                $response = array(
                    "error" => true,
                    "message" => "Error en la consulta", 
                    "data" => $e->getMessage()
                );
            }
        } catch ( QueryException $e ) {
            $response = array(
                "error" => true,
                "message" => "Error en la consulta", 
                "data" => $e->getMessage()
            );
        }

        return response()
            ->json($response, 200)
            ->header("Access-Control-Allow-Origin", "*");
    }

    public function deleteVideos(Request $r) {
        $response = array(
            "error" => false,
            "message" => "Sin error"
        );

        try {
            $LearnVideos = LearnVideos::find($r->id);

            if ( $LearnVideos->delete() ) {
                $response = array(
                    "error" => false,
                    "message" => "Sin error", 
                    "data" => $LearnVideos
                );
            } else {
                $response = array(
                    "error" => true,
                    "message" => "Error en la consulta", 
                    "data" => $e->getMessage()
                );
            }
        } catch ( QueryException $e ) {
            $response = array(
                "error" => true,
                "message" => "Error en la consulta", 
                "data" => $e->getMessage()
            );
        }

        return response()
            ->json($response, 200)
            ->header("Access-Control-Allow-Origin", "*");
    }

    public function getDataCategorias () {
        if ( Request()->header('x-glowme-key') != $_ENV['X_GLOWME_API'] ) { 
            return response()
            ->json([ 
                'error' => true, 
                'message' => "Api key error", 
                'data' => "Acceso no permitido" 
            ], 403)
            ->header("Access-Control-Allow-Origin", "*");
        } else {
            try {
                $categorias = LearnCategories::select("id", "name", DB::Raw("CONCAT( '".$_ENV["URL_PATH"]."', image ) as image"))->get();

                $restResponse = 200;
                $response = array(
                    "error" => false,
                    "message" => "Consulta exitosa", 
                    "data" => $categorias
                );
            } catch (QueryException $e) {
                $restResponse = 500;
                $response = array(
                    "error" => true,
                    "message" => "Error en la consulta", 
                    "data" => $e->getMessage()
                );
            }

            return response()
                ->json($response, $restResponse)
                ->header("Access-Control-Allow-Origin", "*");
        }
    }

    public function getDataCategoriasVideos ( Request $r ) {
        try {
            $categorias = LearnCategories::select("name")->where("id", $r->id)->first()->toArray();
            $videos = LearnVideos::where("i_category", $r->id)->get()->toArray();

            $last = date("Y-m-d", strtotime($videos[0]['updated_at']));
            $today = date("Y-m-d");

            $result = array();
            if ( $videos != null ){
                foreach ( $videos as $item ) {
                    if ( $last != $today ) {
                        preg_match("/\d+/", $item['vimeo'], $idVimeo, PREG_OFFSET_CAPTURE, 0);

                        $headers = array(
                            'Content-Type:application/json',
                            'Authorization: Bearer '. $_ENV['VIMEO_TOKEN']
                        );

                        $curl = curl_init( $_ENV['VIMEO_URL'] . "/" . $idVimeo[0][0] );
                        curl_setopt($curl, CURLOPT_RETURNTRANSFER, 1);
                        curl_setopt($curl, CURLOPT_TIMEOUT, 30);
                        curl_setopt($curl, CURLOPT_FOLLOWLOCATION, 1);
                        curl_setopt($curl, CURLOPT_HTTPHEADER, $headers);
                        $return = curl_exec($curl);
                        curl_close($curl);
                        $arrayResponse = json_decode($return, true);

                        $linkWeb = "";
                        foreach ($arrayResponse['play']['progressive'] as $itemLinkWeb ) {
                            if( $itemLinkWeb['width'] == 1920 ) {
                                $linkWeb = $itemLinkWeb['link'];
                            }
                        }
                        
                        $linkPreview = "";
                        foreach ( $arrayResponse['pictures']["sizes"] as $itemLinkPreview ) {
                            if( $itemLinkPreview['width'] == 1920 ) {
                                $linkPreview = $itemLinkPreview['link'];
                            }
                        }

                        $linkMobile = "";
                        foreach ( $arrayResponse['files'] as $itemLinkMobile ) {
                            if( $itemLinkMobile['quality'] == "hls" ) {
                                $linkMobile = $itemLinkMobile['link'];
                            }
                        }
                    
                        $response = array(
                            "categoria" => $categorias["name"],
                            "title" => $item["title"],
                            "desc" => $item["desc"],
                            "preview" => $linkPreview,
                            "videoWeb" => $linkWeb,
                            "videomobile" => $linkMobile
                        );

                        LearnVideos::where("id", $item["id"])->update([
                            "vimeo" => $idVimeo[0][0], 
                            "preview" => $linkPreview, 
                            "videoWeb" => $linkWeb, 
                            "videomobile" => $linkMobile
                        ]);
                    } else {
                        $response = array(
                            "categoria" => $categorias["name"],
                            "title" => $item["title"],
                            "desc" => $item["desc"],
                            "preview" => $item["preview"],
                            "videoWeb" => $item["videoWeb"],
                            "videomobile" => $item["videoMobile"]
                        );
                    }
                    array_push($result, $response);
                }
            } 

            $restResponse = 200;
            $response = array(
                "error" => false,
                "message" => "Consulta exitosa", 
                "data" => $result
            );
        } catch (QueryException $e) {
            $restResponse = 500;
            $response = array(
                "error" => true,
                "message" => "Error en la consulta", 
                "data" => $e->getMessage()
            );
        }

        return response()
            ->json($response, $restResponse)
            ->header("Access-Control-Allow-Origin", "*");
    }
}
