<?php

namespace App\Http\Controllers\Api\Programs;

use App\Http\Controllers\Controller;
use Illuminate\Http\Request;
use Illuminate\Support\Facades\DB;
use Illuminate\Database\QueryException;
use App\Models\Programs;
use App\Models\Faqs;
use App\Models\Reviews;
use Illuminate\Support\Facades\File;

class ProgramsController extends Controller {
    public function getPrograms() {
        $response = array(
            "error" => false,
            "message" => "Sin error"
        );

        try {
            $programs = Programs::select(
                "programs.id", 
                "programs.name", 
                "programs.status"
            )
            ->orderBy("order")
            ->get();

            $response = array(
                "error" => true,
                "message" => "Sin error", 
                "data" => $programs
            );
        } catch ( QueryException $e ) {
            $response = array(
                "error" => true,
                "message" => "Error en la consulta", 
                "data" => $e->getMessage()
            );
        }

        return response()
            ->json($response, 200)
            ->header("Access-Control-Allow-Origin", "*");
    }

    public function getProgramID($id) {
        $response = array(
            "error" => false,
            "message" => "Sin error"
        );

        try {
            $program = Programs::select(
                "name", 
                "able", 
                "about", 
                "syllabus", 
                "foryou", 
                "benefits", 
                "image", 
                "video", 
                "status"
            )
            ->where("id", $id)
            ->get();

            $faq = Faqs::select(
                "id", 
                "question", 
                "answer", 
                "status"
            )
            ->where("program", $id)
            ->get();

            $review = Reviews::select(
                "id", 
                "stars", 
                "title", 
                "name", 
                "review", 
                "status"
            )
            ->where("program", $id)
            ->get();

            $response = array(
                "error" => false,
                "message" => "Sin error", 
                "data" => [$program, $faq, $review]
            );

        } catch ( QueryException $e ) {
            $response = array(
                "error" => true,
                "message" => "Error en la consulta", 
                "data" => $e->getMessage()
            );
        }

        return response()
            ->json($response, 200)
            ->header("Access-Control-Allow-Origin", "*");
    }

    public function store(Request $r) {
        $response = array(
            "error" => false,
            "message" => "Sin error"
        );

        $response = array(
            "error" => true,
            "message" => "Error en la consulta", 
            "data" => $r->post()
        );

        try {
            $program = new Programs();
            $program->name = $r->name;
            $program->able = $r->able;
            $program->about = $r->about;
            $program->syllabus = $r->syllabus;
            $program->foryou = $r->foryou;
            $program->benefits = $r->benefits;
            $program->video = $r->video;
            $program->status = isset($r->status) ? 1 : 0;

            //Files
            if( $r->has("image") ) {
                $fName = $r->file('image');
                $fileName = MD5($fName . uniqid() . microtime()) . "." . $fName->getClientOriginalExtension();
                $r->file('image')->move("img/programas", $fileName);
                $program->image = "/img/programas/" . $fileName;
            }

            if ( $program->save() ) {

                if ( $r->question[0] != null ){
                    for( $i = 0; $i < count($r->question); $i++ ) {
                        $faq = new Faqs();
                        $faq->program = $program->id;
                        $faq->question = $r->question[$i];
                        $faq->answer = $r->answer[$i];
                        $faq->save();
                    }
                }

                if ( $r->review[0] != null ){
                    for( $i = 0; $i < count($r->review); $i++ ) {
                        $review = new Reviews();
                        $review->program = $program->id;
                        $review->stars = $r->stars[$i];
                        $review->title = $r->title[$i];
                        $review->name = $r->nameReview[$i];
                        $review->review = $r->review[$i];
                        $review->save();
                    }
                }

                $response = array(
                    "error" => false,
                    "message" => "Sin error", 
                    "data" => $program
                );
            } else {
                $response = array(
                    "error" => true,
                    "message" => "Error en la consulta", 
                    "data" => $e->getMessage()
                );
            }
        } catch ( QueryException $e ) {
            $response = array(
                "error" => true,
                "message" => "Error en la consulta", 
                "data" => $e->getMessage()
            );
        }

        return response()
            ->json($response, 200)
            ->header("Access-Control-Allow-Origin", "*");
    }

    public function update(Request $r) {
        $response = array(
            "error" => false,
            "message" => "Sin error"
        );

        try {
            $program = Programs::find($r->id);
            $program->name = $r->name;
            $program->able = $r->able;
            $program->about = $r->about;
            $program->syllabus = $r->syllabus;
            $program->foryou = $r->foryou;
            $program->benefits = $r->benefits;
            $program->video = $r->video;
            $program->status = isset($r->status) ? 1 : 0;

            //Files
            if( $r->has("image") ) {
                //Borrar Banner
                $path = ltrim($program->image, '/');
                if ( File::exists($path) ) {
                    unlink( $path );
                }
                
                $fName = $r->file('image');
                $fileName = MD5($fName . uniqid() . microtime()) . "." . $fName->getClientOriginalExtension();
                $r->file('image')->move("img/programas", $fileName);
                $program->image = "/img/programas/" . $fileName;
            } 

            if ( $program->save() ) {
                // Borra registros viejos

                if ( $r->question != null ){
                    $faqs = Faqs::where("program", $r->id)->delete();
                    for( $i = 0; $i < count($r->question); $i++ ) {
                        $faq = new Faqs();
                        $faq->program = $program->id;
                        $faq->question = $r->question[$i];
                        $faq->answer = $r->answer[$i];
                        $faq->save();
                    }
                }

                if ( $r->review != null ){
                    $reviews = Reviews::where("program", $r->id)->delete();
                    for( $i = 0; $i < count($r->review); $i++ ) {
                        $review = new Reviews();
                        $review->program = $program->id;
                        $review->stars = $r->stars[$i];
                        $review->title = $r->title[$i];
                        $review->name = $r->nameReview[$i];
                        $review->review = $r->review[$i];
                        $review->save();
                    }
                }

                $response = array(
                    "error" => false,
                    "message" => "Sin error", 
                    "data" => $program
                );
            } else {
                $response = array(
                    "error" => true,
                    "message" => "Error en la consulta", 
                    "data" => $e->getMessage()
                );
            }
        } catch ( QueryException $e ) {
            $response = array(
                "error" => true,
                "message" => "Error en la consulta", 
                "data" => $e->getMessage()
            );
        }

        return response()
            ->json($response, 200)
            ->header("Access-Control-Allow-Origin", "*");
    }

    public function delete(Request $r) {
        $response = array(
            "error" => false,
            "message" => "Sin error"
        );

        try {
            $program = Programs::find($r->id);

            if ( $program->delete() ) {
                
                $faqs = Faqs::where("program", $r->id)->delete();
                $reviews = Reviews::where("program", $r->id)->delete();

                $response = array(
                    "error" => false,
                    "message" => "Sin error", 
                    "data" => $program
                );
            } else {
                $response = array(
                    "error" => true,
                    "message" => "Error en la consulta", 
                    "data" => $e->getMessage()
                );
            }
        } catch ( QueryException $e ) {
            $response = array(
                "error" => true,
                "message" => "Error en la consulta", 
                "data" => $e->getMessage()
            );
        }

        return response()
            ->json($response, 200)
            ->header("Access-Control-Allow-Origin", "*");
    }
}
