<?php

namespace App\Http\Controllers\Api\Users;

use App\Http\Controllers\Controller;
use Illuminate\Http\Request;
use Illuminate\Support\Facades\DB;
use Illuminate\Database\QueryException;
use App\Models\UserProfiles;
use App\Models\Users;
use Illuminate\Support\Facades\File;

class UsersController extends Controller {
    public function getUsers() {
        $response = array(
            "error" => false,
            "message" => "Sin error"
        );

        try {
            $profiles = DB::table("users")
            ->select(
                "users.id", 
                "users.name", 
                "users.email", 
                "profile_image", 
                "users.status", 
                DB::Raw("user_profiles.name as profileName"), 
                "user_profiles.permits"
            )
            ->join("user_profiles", "users.user_profile_id", "=", "user_profiles.id")
            ->whereNull("users.deleted_at")
            ->whereNull("user_profiles.deleted_at")
            ->get();

            $response = array(
                "error" => true,
                "message" => "Sin error", 
                "data" => $profiles
            );
        } catch ( QueryException $e ) {
            $response = array(
                "error" => true,
                "message" => "Error en la consulta", 
                "data" => $e->getMessage()
            );
        }

        return response()
            ->json($response, 200)
            ->header("Access-Control-Allow-Origin", "*");
    }

    public function getUserID($id) {
    	$response = array(
            "error" => false,
            "message" => "Sin error"
        );

        try {
            $user = DB::table("users")
            ->select(
        		"users.name", 
                "users.email", 
                "users.password", 
                "users.user_profile_id", 
                "users.profile_image", 
                "users.status", 
                DB::Raw("user_profiles.name AS proflieName")
        	)
        	->join("user_profiles", "users.user_profile_id", "=", "user_profiles.id")
            ->where("users.id", $id)
        	->whereNull("users.deleted_at")
            ->whereNull("user_profiles.deleted_at")
            ->get();

			$response = array(
                "error" => false,
                "message" => "Sin error", 
                "data" => $user
            );

        } catch ( QueryException $e ) {
            $response = array(
                "error" => true,
                "message" => "Error en la consulta", 
                "data" => $e->getMessage()
            );
        }

        return response()
            ->json($response, 200)
            ->header("Access-Control-Allow-Origin", "*");
    }

    public function store(Request $r) {
        $response = array(
            "error" => false,
            "message" => "Sin error"
        );

        try {
        	$user = new Users();
        	$user->name = $r->name;
        	$user->email = $r->mail;
            $user->password = bcrypt($r->pass);
            $user->position = 1;
            $user->user_profile_id = $r->perfil;
            $user->status = 1;

            //Files
            if( $r->has("image") ) {
                $fName = $r->file('image');
                $fileName = MD5($fName . uniqid() . microtime()) . "." . $fName->getClientOriginalExtension();
                $r->file('image')->move("img/profile", $fileName);
                $user->profile_image = "/img/profile/" . $fileName;
            }

			if ( $user->save() ) {
	            $response = array(
	                "error" => false,
	                "message" => "Sin error", 
	                "data" => $user
	            );
	        } else {
	        	$response = array(
	                "error" => true,
	                "message" => "Error en la consulta", 
	                "data" => $e->getMessage()
	            );
	        }
        } catch ( QueryException $e ) {
            $response = array(
                "error" => true,
                "message" => "Error en la consulta", 
                "data" => $e->getMessage()
            );
        }

        return response()
            ->json($response, 200)
            ->header("Access-Control-Allow-Origin", "*");
    }

    public function update(Request $r) {
        $response = array(
            "error" => false,
            "message" => "Sin error"
        );

        try {
        	$user = Users::find($r->id);
        	$user->name = $r->name;
            $user->email = $r->mail;
            $user->user_profile_id = $r->perfil;
            
            if ( $r->pass != "" ) {
                $user->password = bcrypt($r->pass);
            }

            //Files
            if( $r->has("image") ) {
                //Borrar Banner
                $path = ltrim($user->profile_image, '/');
                if ( File::exists($path) ) {
                    unlink( $path );
                }

                $fName = $r->file('image');
                $fileName = MD5($fName . uniqid() . microtime()) . "." . $fName->getClientOriginalExtension();
                $r->file('image')->move("img/profile", $fileName);
                $user->profile_image = "/img/profile/" . $fileName;
            } 

			if ( $user->save() ) {
	            $response = array(
	                "error" => false,
	                "message" => "Sin error", 
	                "data" => $user
	            );
	        } else {
	        	$response = array(
	                "error" => true,
	                "message" => "Error en la consulta", 
	                "data" => $e->getMessage()
	            );
	        }
        } catch ( QueryException $e ) {
            $response = array(
                "error" => true,
                "message" => "Error en la consulta", 
                "data" => $e->getMessage()
            );
        }

        return response()
            ->json($response, 200)
            ->header("Access-Control-Allow-Origin", "*");
    }

    public function delete(Request $r) {
        $response = array(
            "error" => false,
            "message" => "Sin error"
        );

        try {
        	$user = Users::find($r->id);

			if ( $user->delete() ) {
                //Borrar pdf
                $path = ltrim($user->profile_image, '/');
                if ( File::exists($path) ) {
                    unlink( $path );
                }

	            $response = array(
	                "error" => false,
	                "message" => "Sin error", 
	                "data" => $user
	            );
	        } else {
	        	$response = array(
	                "error" => true,
	                "message" => "Error en la consulta", 
	                "data" => $e->getMessage()
	            );
	        }
        } catch ( QueryException $e ) {
            $response = array(
                "error" => true,
                "message" => "Error en la consulta", 
                "data" => $e->getMessage()
            );
        }

        return response()
            ->json($response, 200)
            ->header("Access-Control-Allow-Origin", "*");
    }
}
