<?php

namespace App\Http\Controllers\Api\Workouts;

use App\Http\Controllers\Controller;
use Illuminate\Http\Request;
use Illuminate\Support\Facades\DB;
use Illuminate\Database\QueryException;
use Illuminate\Support\Facades\File;

use App\Models\Workouts;
use App\Models\WorkoutsVideos;

class WorkoutsController extends Controller {
    public function getWorkouts() {
        $response = array(
            "error" => false,
            "message" => "Sin error"
        );

        try {
            $workouts = Workouts::select(
                "workouts.id", 
                "workouts.program", 
                "workouts.coach"
            )
            ->get();

            $response = array(
                "error" => true,
                "message" => "Sin error", 
                "data" => $workouts
            );
        } catch ( QueryException $e ) {
            $response = array(
                "error" => true,
                "message" => "Error en la consulta", 
                "data" => $e->getMessage()
            );
        }

        return response()
            ->json($response, 200)
            ->header("Access-Control-Allow-Origin", "*");
    }

    public function getWorkoutID($id) {
        $response = array(
            "error" => false,
            "message" => "Sin error"
        );

        try {
            $workout = Workouts::select(
                "program", 
                "banner", 
                "info", 
                "coach", 
                "coach_pic", 
                "coach_info", 
                "duration", 
                "by_week", 
                "class_time"
            )
            ->where("id", $id)
            ->get();

            $response = array(
                "error" => false,
                "message" => "Sin error", 
                "data" => $workout
            );

        } catch ( QueryException $e ) {
            $response = array(
                "error" => true,
                "message" => "Error en la consulta", 
                "data" => $e->getMessage()
            );
        }

        return response()
            ->json($response, 200)
            ->header("Access-Control-Allow-Origin", "*");
    }

    public function store(Request $r) {
        $response = array(
            "error" => false,
            "message" => "Sin error"
        );

        try {
            $workout = new Workouts();
            $workout->program = $r->program;
            $workout->info = $r->info;
            $workout->coach = $r->coach;
            $workout->coach_info = $r->coach_info;
            $workout->by_week = $r->by_week;
            $workout->class_time = $r->class_time;

            //Files
            if( $r->has("banner") ) {
                $fName = $r->file('banner');
                $fileName = MD5($fName . uniqid() . microtime()) . "." . $fName->getClientOriginalExtension();
                $r->file('banner')->move("img/workouts", $fileName);
                $workout->banner = "/img/workouts/" . $fileName;
            }

            if( $r->has("coach_pic") ) {
                $fName = $r->file('coach_pic');
                $fileName = MD5($fName . uniqid() . microtime()) . "." . $fName->getClientOriginalExtension();
                $r->file('coach_pic')->move("img/workouts", $fileName);
                $workout->coach_pic = "/img/workouts/" . $fileName;
            }

            if ( $workout->save() ) {
                $response = array(
                    "error" => false,
                    "message" => "Sin error", 
                    "data" => $workout
                );
            } else {
                $response = array(
                    "error" => true,
                    "message" => "Error en la consulta", 
                    "data" => $e->getMessage()
                );
            }
        } catch ( QueryException $e ) {
            $response = array(
                "error" => true,
                "message" => "Error en la consulta", 
                "data" => $e->getMessage()
            );
        }

        return response()
            ->json($response, 200)
            ->header("Access-Control-Allow-Origin", "*");
    }

    public function update(Request $r) {
        $response = array(
            "error" => false,
            "message" => "Sin error"
        );

        try {
            $workout = Workouts::find($r->id);
            $workout->program = $r->program;
            $workout->info = $r->info;
            $workout->coach = $r->coach;
            $workout->coach_info = $r->coach_info;
            $workout->duration = $r->duracion;
            $workout->by_week = $r->by_week;
            $workout->class_time = $r->class_time;

            //Files
            if( $r->has("banner") ) {
                //Borrar Banner
                $path = ltrim($workout->banner, '/');
                if ( File::exists($path) ) {
                    unlink( $path );
                }
                $fName = $r->file('banner');
                $fileName = MD5($fName . uniqid() . microtime()) . "." . $fName->getClientOriginalExtension();
                $r->file('banner')->move("img/workouts", $fileName);
                $workout->banner = "/img/workouts/" . $fileName;
            }

            if( $r->has("coach_pic") ) {
                //Borrar Banner
                $path = ltrim($workout->coach_pic, '/');
                if ( File::exists($path) ) {
                    unlink( $path );
                }
                $fName = $r->file('coach_pic');
                $fileName = MD5($fName . uniqid() . microtime()) . "." . $fName->getClientOriginalExtension();
                $r->file('coach_pic')->move("img/workouts", $fileName);
                $workout->coach_pic = "/img/workouts/" . $fileName;
            }

            if ( $workout->save() ) {
                $response = array(
                    "error" => false,
                    "message" => "Sin error", 
                    "data" => $workout
                );
            } else {
                $response = array(
                    "error" => true,
                    "message" => "Error en la consulta", 
                    "data" => $e->getMessage()
                );
            }
        } catch ( QueryException $e ) {
            $response = array(
                "error" => true,
                "message" => "Error en la consulta", 
                "data" => $e->getMessage()
            );
        }

        return response()
            ->json($response, 200)
            ->header("Access-Control-Allow-Origin", "*");
    }

    public function delete(Request $r) {
        $response = array(
            "error" => false,
            "message" => "Sin error"
        );

        try {
            $workout = Workouts::find($r->id);
            $videos = $workout->get();

            if ( $workout->delete() ) {
                //Borrar Banner
                $path = ltrim($videos[0]->banner, '/');
                if ( File::exists($path) ) {
                    unlink( $path );
                }

                //Borrar Banner
                $path = ltrim($videos[0]->coach_pic, '/');
                if ( File::exists($path) ) {
                    unlink( $path );
                }

                WorkoutsVideos::where('i_workout', $videos[0]->id)->delete();

            } else {
                $response = array(
                    "error" => true,
                    "message" => "Error en la consulta", 
                    "data" => $e->getMessage()
                );
            }
        } catch ( QueryException $e ) {
            $response = array(
                "error" => true,
                "message" => "Error en la consulta", 
                "data" => $e->getMessage()
            );
        }

        return response()
            ->json($response, 200)
            ->header("Access-Control-Allow-Origin", "*");
    }
}
