<?php
// https://stripe.com/docs/api/subscriptions/create
// https://stripe.com/docs/billing/subscriptions/paypal
namespace App\Http\Controllers\Stripe;

use App\Http\Controllers\Controller;
use Illuminate\Http\Request;
use Illuminate\Support\Facades\DB;
use Illuminate\Database\QueryException;
use App\Models\UserProfiles;
use App\Models\Users;
use App\Models\Clients;
use App\Models\Charges;
use App\Models\Suscriptions;

use PHPMailer\PHPMailer\PHPMailer;
use PHPMailer\PHPMailer\SMTP;
use PHPMailer\PHPMailer\Exception;

class StripeController extends Controller {
    public function index( ) {
        return view('stripe');
    }

    public function create() {
        $stripe = new \Stripe\StripeClient($_ENV['STRIPE_SECRET']);

        function calculateOrderAmount(array $items): int {
            // Replace this constant with a calculation of the order's amount
            // Calculate the order total on the server to prevent
            // people from directly manipulating the amount on the client
            return $items[0]->amount;
        }

        header('Content-Type: application/json');

        try {
            // retrieve JSON from POST body
            $jsonStr = file_get_contents('php://input');
            $jsonObj = json_decode($jsonStr);

            // Create a PaymentIntent with amount and currency
            $paymentIntent = $stripe->paymentIntents->create([
                'amount' => calculateOrderAmount($jsonObj->items),
                'currency' => 'mxn',
                // In the latest version of the API, specifying the `automatic_payment_methods` parameter is optional because Stripe enables its functionality by default.
                'automatic_payment_methods' => [
                    'enabled' => true,
                ],
            ]);

            $output = [
                'clientSecret' => $paymentIntent->client_secret,
            ];

            echo json_encode($output);
        } catch (Error $e) {
            http_response_code(500);
            echo json_encode(['error' => $e->getMessage()]);
        }
    }

    public function createCustomer( Request $r ) {
        $stripe = new \Stripe\StripeClient($_ENV['STRIPE_SECRET']);
        
        $createReponse = $stripe->customers->create([
            'email' => 'mauricio.salinas@metodika.mx',
            'name' => 'Mauricio Salinas',
            'phone' => '528182522569',
            'description' => 'My First Test Customer (created for API docs at https://www.stripe.com/docs/api)',
        ]);

        // Save the $createReponse->id in clients
    }

    public function getCustomer () {
        $stripe = new \Stripe\StripeClient($_ENV['STRIPE_SECRET']);
        
        $createReponse = $stripe->customers->retrieve(
          'cus_Ol6WlCHLOc7d29',
          []
        );

        dd( $createReponse );
    }

    public function updateCustomer( Request $r ) {
        $stripe = new \Stripe\StripeClient($_ENV['STRIPE_SECRET']);

        $createReponsePayment = $stripe->paymentMethods->attach(
            $r->payment,
            ['customer' => 'cus_Ol6WlCHLOc7d29']
        );

        $createReponsePaymentDefault = $stripe->customers->update(
          'cus_Ol6WlCHLOc7d29',
          ['invoice_settings' => ['default_payment_method' => $createReponsePayment->id]]
        );

        // Save the $createReponse->id in clients
    }

    public function customerSubscribe () {
        $stripe = new \Stripe\StripeClient($_ENV['STRIPE_SECRET']);
        
        $createReponse = $stripe->subscriptions->create([
            'customer' => 'cus_Ol6WlCHLOc7d29',
            'items' => [
                ['price' => 'price_1NxdENJUadaofAxut9kTkk37'],
            ],
        ]);

        dd($createReponse);
    }

    public function createProducto () { //Crear Producto
        $stripe = new \Stripe\StripeClient($_ENV['STRIPE_SECRET']);
        
        $createReponseMensual = $stripe->products->create([
            'name' => 'Plan Mensual',
            'description' => 'Costo del plan $799'
        ]);

        $createReponseTrimestral = $stripe->products->create([
            'name' => 'Plan Trimestral',
            'description' => 'Costo del plan $1782'
        ]);

        $createReponseSemestral = $stripe->products->create([
            'name' => 'Plan Semestral',
            'description' => 'Costo del plan $3355'
        ]);

        dd($createReponseMensual, $createReponseTrimestral, $createReponseSemestral);

        // Save the $createReponse->id in clients
    }

    public function createPrice () { //Crear Precio
        $stripe = new \Stripe\StripeClient($_ENV['STRIPE_SECRET']);
        
        $createReponseMensual = $stripe->prices->create([
            'unit_amount' => 79900,
            'currency' => 'mxn',
            'recurring' => ['interval' => 'month', 'interval_count' => 1],
            'product' => 'prod_Oqm8aMMKbijGPc'
        ]);

        $createReponseTrimestral = $stripe->prices->create([
            'unit_amount' => 178200,
            'currency' => 'mxn',
            'recurring' => ['interval' => 'month', 'interval_count' => 3],
            'product' => 'prod_Oqm8PtVV6nI97f'
        ]);

        $createReponseSemestral = $stripe->prices->create([
            'unit_amount' => 335500,
            'currency' => 'mxn',
            'recurring' => ['interval' => 'month', 'interval_count' => 6],
            'product' => 'prod_Oqm8VXrAAIuljy'
        ]);

        dd($createReponseMensual, $createReponseTrimestral, $createReponseSemestral);

        // Save the $createReponse->id in clients
    }

    public function stripeWebhook() {
        // webhook.php
        //
        // Use this sample code to handle webhook events in your integration.
        //
        // 1) Paste this code into a new file (webhook.php)
        //
        // 2) Install dependencies
        //   composer require stripe/stripe-php
        //
        // 3) Run the server on http://localhost:4242
        //   php -S localhost:4242

        // The library needs to be configured with your account's secret key.
        // Ensure the key is kept out of any version control system you might be using.
        $stripe = new \Stripe\StripeClient($_ENV['STRIPE_SECRET']);

        // This is your Stripe CLI webhook secret for testing your endpoint locally.
        $endpoint_secret = $_ENV['WEBHOOK_SECRET'];

        $payload = @file_get_contents('php://input');
        $sig_header = $_SERVER['HTTP_STRIPE_SIGNATURE'];
        $event = null;

        try {
          $event = \Stripe\Webhook::constructEvent(
            $payload, $sig_header, $endpoint_secret
          );
        } catch(\UnexpectedValueException $e) {
          // Invalid payload
          http_response_code(400);
          exit();
        } catch(\Stripe\Exception\SignatureVerificationException $e) {
          // Invalid signature
          http_response_code(400);
          exit();
        }

        // Handle the event
        switch ($event->type) {
          case 'charge.failed':
            $charge = $event->data->object->object;
            $date = date("Y-m-d H:i:s", $event->data->object->created);
            $paid = $event->data->object->paid;
            $amount = $event->data->object->amount;
            $customer = $event->data->object->customer;

            $Client = Clients::select("id", "fullname", "email", "suscription_plan")->where("stripe_source", $customer)->first(); 

            if ( $Client != null ) {
                Clients::where("stripe_source", $customer)->update(["suscription_cancel_at" => $date, "status" => -2]);

                $Charges = new Charges();
                $Charges->i_client = $Client['id'];
                $Charges->i_subscription = $Client['suscription_plan'];
                $Charges->vc_amount = $amount;
                $Charges->status = "Failed";
                $Charges->save();

                $view = view('emails/pay-fail', ["name" => $Client['fullname']])->render();

                $mail = new PHPMailer(true);
                $mail->Host = "smtp.gmail.com";
                $mail->Port       = 587;
                $mail->SMTPSecure = '';
                $mail->SMTPAuth   = true;
                $mail->IsHTML(true);
                $mail->CharSet = 'UTF-8';

                $mail->Username = "hola@glowmeapp.com";
                $mail->Password = "ckqbyhujrpdesrrk";
                $mail->SetFrom('hola@glowmeapp.com', 'Hola GlowMe');
                $mail->addReplyTo('hola@glowmeapp.com', 'Hola GlowMe');

                $mail->Subject = "GlowMe Cobro Fallido";
                $mail->Body = $view; 
                $mail->AddAddress($Client['email'], "{$Client['fullname']}");
                $mail->addBCC('mauricio.salinas@metodika.mx', 'Leads');
                $mail->addBCC('leads@metodika.mx', 'Leads');
                $mail->addBCC('carlos.betancourt@metodika.mx', 'Carlos');
                $mail->addBCC('clara.viera@metodika.mx', 'Clara');

                $mail->Send();
            }
            break;
          case 'charge.succeeded':
            $charge = $event->data->object->object;
            $date = date("Y-m-d H:i:s", $event->data->object->created);
            $paid = $event->data->object->paid;
            $amount = $event->data->object->amount;
            $customer = $event->data->object->customer;

            $Client = Clients::select("id", "fullname", "email", "suscription_plan")->where("stripe_source", $customer)->first();

            if ( $Client != null ) {
                Clients::where("stripe_source", $customer)->update(["last_charge" => $date, "status" => 1]);

                $Charges = new Charges();
                $Charges->i_client = $Client['id'];
                $Charges->i_subscription = $Client['suscription_plan'];
                $Charges->vc_amount = $amount;
                $Charges->status = "Success";
                $Charges->save();

                $Suscriptions = Suscriptions::select("name", "cost", DB::Raw("
                    CASE
                        WHEN id = 1 THEN cost/1
                        WHEN id = 2 THEN cost/3
                        WHEN id = 3 THEN cost/6
                    END
                as plan"))->where("id", $Client['suscription_plan'])->first();

                $date_future = strtotime('+30 day', strtotime(date("Y-m-d")));

                $view = view('emails/pay-success', ["name" => $Client['fullname'], "data" => array("name" => $Suscriptions['name'], "cost" => $Suscriptions['cost'], "plan" => $Suscriptions['plan']), "next" => date("d", $date_future) . " / " . date("m", $date_future) . " / " . date("Y", $date_future) ])->render();

                $mail = new PHPMailer(true);
                $mail->Host = "smtp.gmail.com";
                $mail->Port       = 587;
                $mail->SMTPSecure = '';
                $mail->SMTPAuth   = true;
                $mail->IsHTML(true);
                $mail->CharSet = 'UTF-8';

                $mail->Username = "hola@glowmeapp.com";
                $mail->Password = "ckqbyhujrpdesrrk";
                $mail->SetFrom('hola@glowmeapp.com', 'Hola GlowMe');
                $mail->addReplyTo('hola@glowmeapp.com', 'Hola GlowMe');

                $mail->Subject = "GlowMe Renovación de Plan";
                $mail->Body = $view; 
                $mail->AddAddress($Client['email'], "{$Client['fullname']}");
                $mail->addBCC('mauricio.salinas@metodika.mx', 'Leads');
                $mail->addBCC('leads@metodika.mx', 'Leads');
                $mail->addBCC('carlos.betancourt@metodika.mx', 'Carlos');
                $mail->addBCC('clara.viera@metodika.mx', 'Clara');

                $mail->Send();
            }
            break;
          default:
            echo 'Received unknown event type ' . $event->type;
        }

        http_response_code(200);
    }
}

// Preguntar por registrar log de cobros
// Probar con customer